#!user/bin/python
# _*_ coding: utf-8 _*_
# @Author      :   Jesper
# @Time        :   2021/7/20  11:13
# @Description :

import json
import pickle
from nlp_utils_ch.utils.logger import log

def save_pickle(data, file_path):
	'''
	保存成pickle文件
	:param data:
	:param file_name:
	:param pickle_path:
	:return:
	'''
	with open(file_path, 'wb') as f:
		pickle.dump(data, f)


def load_pickle(input_file):
	'''
	读取pickle文件
	:param pickle_path:
	:param file_name:
	:return:
	'''
	with open(str(input_file), 'rb') as f:
		data = pickle.load(f)
	return data


def save_json(data, file_path, ensure_ascii=False, indent=4):
	'''
	保存成json文件
	:param data:
	:param json_path:
	:param file_name:
	:return:
	'''

	with open(str(file_path), 'w') as f:
		if indent == 0:
			json.dump(data, f, ensure_ascii=ensure_ascii)
		else:
			json.dump(data, f, ensure_ascii=ensure_ascii, indent=4)


def load_json(file_path):
	'''
	加载json文件
	:param json_path:
	:param file_name:
	:return:
	'''
	with open(str(file_path), 'r') as f:
		data = json.load(f)
	return data


## 解决excel直接复制到csv，因换行,tab符号导致顺序错乱
def excel_to_csv(src_path, tgt_path, replace_chars=["\n", "\t"],
                header=True,
                content_col="content",
                 src_file_type="excel"):
	""":arg
	src_file_type: excel/csv
	"""
	# import 写在函数内，避免其它调用时
	import pandas as pd
	if src_file_type == "excel":
		raw_data = pd.read_excel(src_path, encoding="utf-8", errors='ignore')
	elif src_file_type == "csv":
		raw_data = pd.read_csv(src_path, encoding="utf-8")
	else:
		raise NotImplemented
	log.info("raw_data count:{}".format(len(raw_data)))
	col_names = raw_data.columns.values
	res_data = list()
	for i in range(len(raw_data)):
		line_list = list()
		for _col_name in col_names:
			_value = str(raw_data[_col_name][i])
			if _col_name == content_col:
				for _char in replace_chars:
					_value = _value.replace(_char, " ")
			line_list.append(_value)
		res_data.append("\t".join(line_list))

	with open(tgt_path, "w") as f:
		for _line in res_data:
			# for _char in replace_chars:
			_line = _line.replace("\n", " ")
			f.write(_line + "\n")


def json_format(src_path, tgt_path=None):
	data = load_json(src_path)
	if isinstance(data, list):
		log.info("data num:{}".format(len(data)))

	if tgt_path:
		save_json(data, tgt_path, ensure_ascii=False, indent=4)
	else:
		save_json(data, src_path, ensure_ascii=False, indent=4)



# -*- coding: utf-8 -*-
"""
   Author :        Jesper
   Date：          2020/2/25 17:27
   Description :
   Changed by:
"""

# 统一文本编码，可用于文本匹配时，避免因全角半角导致匹配不致的问题
def strQ2B(ustring):
    """全角转半角"""
    rstring = ""
    for uchar in ustring:
        inside_code=ord(uchar)
        if inside_code == 12288:                              #全角空格直接转换
            inside_code = 32
        elif (inside_code >= 65281 and inside_code <= 65374): #全角字符（除空格）根据关系转化
            inside_code -= 65248
        rstring += unichr(inside_code)
    return rstring



#!user/bin/python
# _*_ coding: utf-8 _*_
# @Author      :   Jesper
# @Time        :   2020/8/18 15:34
# @Description :


def print_parameters(model):
	params = list(model.parameters())
	k = 0
	for i in params:
		l = 1
		print("该层的结构：" + str(list(i.size())))
		for j in i.size():
			l *= j
		print("该层参数和：" + str(l))
		k = k + l
	print("总参数数量和：" + str(k))



# -*- coding: utf-8 -*-
"""
   Author :        Jesper
   Date：          2020/2/24 20:49
   Description :
   Changed by:
"""


def test_flast_text():
	from flash_text.keyword import KeywordProcessor

	text="这只是一个测试用例"
	words=["测试", "用例"]
	keyword_processor = KeywordProcessor()
	for word in words:
		keyword_processor.add_keyword(word, "123")
	keywords_found = keyword_processor.extract_keywords(text)
	print(keywords_found)

def test_zhconv():
	from zhconv import convert

	# 繁体转简体
	print(convert('我幹什麼不干你事。', 'zh-cn'))
	# 简体转繁体
	print(convert('人体内存在很多微生物', 'zh-tw'))

def test_char_judgement():
	from char_judgement.char_judgement import is_contain_chinese, is_all_chinese, is_ch_digit_letter, len_of_ch_digit_letter, is_all_letter

	res_1 = is_contain_chinese("abc测试一下")
	res_2 = is_contain_chinese("abc")
	print(res_1)
	print(res_2)
	res_3 = is_all_chinese("测试一下")
	res_4 = is_all_chinese("abc测试一下")
	print(res_3)
	print(res_4)
	res_5 = is_ch_digit_letter("123abc测试一下")
	res_6 = is_ch_digit_letter("1")
	print(res_5)
	print(res_6)
	print(len_of_ch_digit_letter("..123abc测试一下a.."))

	res_7 = is_all_letter("abc")
	print(res_7)
	res_8 = is_all_letter("abc12")
	print(res_8 )

def test_scel_to_txt():
	from scel_to_txt.scel2txt import convert_scel_to_txt

	in_path = "your/scel/path/dir"
	out_path = "your/txt/path"
	convert_scel_to_txt(in_path, out_path)

def test_text_clean():
	# from text_clean.filter_text import filter_text_between_bracket, bracket_words_process
	# text = "()你好有东西, 挂精卫哪个位置？_(非常疑惑)"
	# text = "【女武神】丽塔"
	# text = "「影骑士·月轮」作战指南"
	# # res = filter_text_between_bracket(text)
	# res = bracket_words_process(text, is_filter=False)
	# print(res)
	from text_clean.text_tokenize import token_en_number_seq_words
	token_en_number_seq_words("add今天ab真cd4不错ab")

def test_result_analysis():
	# from result_analysis.classification_result import total_accuracy, evaluation_metrics, classification_report
	# y_true = [1, 0, 0, 1, 1, 0, 1, -1, -1]
	# y_pred = [1, 0, 0, 0, 1, 0, 1, -1, 1]
	# res = total_accuracy(y_true, y_pred)
	# print(res)
	# res = evaluation_metrics(y_true, y_pred)
	# print(res)
	#
	# res = classification_report(y_true, y_pred)
	# print(res)

	# rouge metric
	import time
	import sys
	sys.setrecursionlimit(500)
	from result_analysis.rouge_metric import Rouge
	rouge = Rouge(metrics=["rouge-2"], exclusive=False)
	start_time = time.time()
	a = "今天天气不错"
	b = "欢迎新入职的休伯利安舰长，相信各位在经过角色养成篇后以经对(清洗甲板)有了初步了解，接下来就让我来详细教各位舰长如何(空中劈叉)啦～角色装备篇【悬赏委托】(必肝)武器:超电磁手炮☆古老的月光毕业武器，已经基本没有实用价值。不过是超限电磁炮的前置要素（解锁图鉴），依旧可以成为新的毕业武器武器:磁暴斩☆古老的鬼铠毕业武器，攻击自带特效，可惜已经基本没有实战价值。不过是超限磁暴斩的前置要素（解锁图鉴），并且强力推荐超限为重磁暴，使用频率极高武器:妖刀·赤染樱☆真炎幸魂标准的毕业武器，加伤稳定，武器主动可刷新分支的易伤时间（分支自身无法刷新），提高易伤覆盖率。超限武器相较其提升明显，强度有保证武器:百手巨人初型☆目前来说已很少作为输出武器使用，主要用于时空控制并施加脆弱效果。超限后成功实现量变到质变，进而在实战中打出各种战场高分。同时也是山吹的好伙伴，补足脆弱的同时，还实现了延迟角色离场 → 延迟触发BUFF加成☆俗称沙皇中，根据连击数增加无限制的物理伤害，效果优秀☆在满生命情况下可大幅提升物理伤害的通用物理圣痕，不适合持续流血的量子深渊及迪拉克之☆大幅提升对麻痹敌人的元素伤害，在对可麻痹敌人的作战中发挥出色，对其他无法麻痹敌人的作战中等于白板圣痕。☆大幅提升对点燃敌人的元素伤害，目前点燃手段多样，很多BOSS都可点燃，加伤很容易生效。☆俗称艳后下，与苍玄中相似，大幅提升对麻痹敌人的全伤害，因此也可以给物理角色使用，在对可麻痹敌人的作战中发挥出色。☆可显著缩短武器主动CD。特别的，其下位替代——德丽莎·起源（中）已经能够满足日常战斗的基本需求，齐格飞（中）只有在极限竞速的情况下，才能展现出其优势。☆中位用于给圣女快速回复SP（圣女闪避技带有流血效果）。☆可提供雷电易伤效果，但需要启动时间。整体加成偏低，而且还需要启动时间。目前有超限百手延迟驻场的骚套路，但具体实用性不算很高。☆麻痹效果长，CD短，整体覆盖率高，常由月光携带。但在超限电磁炮出现后，武器带来的长时间麻痹效果逐渐取代了开普勒（中）。在没有超限电磁炮之前，推荐月光采用：开普勒（上、中） 艳后（下）的搭配。☆对群效果好，对单弱势。上位限制较多，因此实际加伤能力一般；中位为伤害倍率提升，非常吃角色自身加成；下位为核心单件，减速/冻结的敌人越多，加伤最足。3件套提供高额冰伤加成，因此通常还是成套使用。地图装☆在BOSS战中（单个敌人），所有角色均可使用。中位也算是不错的火伤圣痕（白核装），因为主流的：玫瑰，真红，荣光，其主要输出方式都包含有蓄力攻击☆可提供高额的闪避技冷缩效果☆整体是一套依赖全局时空，才能触发强力效果的圣痕。其中中位的价值最大，可提供高额暴击率，推荐优先锻造（并且也只能获取一个）。上位一般用于快速破盾，2件套可加快圣女的SP回复速度，均主要在战场竞速中使用☆下位非常契合增幅迅羽（迅雷）闪避接大招的输出机制，可提供120%的超高雷伤加成，主要用于BOSS竞速（闪避可控）☆各单件效果一般，主要牛逼在2件套：在仅携带上、中2件套时，只会混出橙色，使目标受到的物理伤害提升30% 。虽然只持续4秒，但已足够部分物理爆发类角色输出（比如增幅游侠）。在合理使用的情况下，其辅助效果会强过古斯塔夫☆一般由辅助携带，虽然10%的概率较低，但18%的全伤害提升也算对的起这个概率，通常是凹战场高分才会使用的圣痕。特别的，该圣痕的词条为“攻击时”，没有限制攻击类型，因此，例如：神恩大招、鬼角主动这样，能后台造成伤害还具备高攻击段数的效果时，暴食加成的期望值会大幅提高，会优于特斯拉（上）等圣痕。(注4.1版本糖从做，会重写)☆极其廉价的白核圣痕。数值高，无需启动是其一大优势。缺点是杀敌会增加自身受到的伤害，不太适合深渊群怪，倒是很适合战场☆后台每秒恢复sp，上场后sp快速衰减，通常有两种携带情况，① ：雪地大招物理易伤兼控制，战车大招远程物理易伤兼聚怪。平时挂后台，SP够了就可以切出来打一波爆发。特别提醒：因为上场后会快速消耗SP，因此按大招的手要快，不然可能SP就不足了。②：懒惰下 血舞。通常用于记忆战场☆一般用于辅助，2件套“下场后全队全伤 10%”的效果非常实用，加成无限制且触发非常简单。中位又可额外提升队友的火元素伤害，整体就辅助火属性主C来说，非常给力。炽翎可穿戴3件套辅助，兼顾输出。☆可作莎士比亚（下）的替代品☆优秀的物理过渡圣痕，上位物理加伤，中位暴击加伤，下位提升暴击率的同时还能回复一定的SP。可根据自身的不足（比如缺中位物理），选择合适的部位过渡。☆并不廉价的雷伤过渡圣痕，上、中一般；下位较为通用，且对于缺少全伤的角色来说提升不错。圣痕整体的伤害加成偏低，相较爱迪生提升有限，且它是白核圣痕，所以并不廉价，值得换的也就下位。☆优秀的近战物理过渡圣痕，上位花瓶，中、下位较为实用，2件套效果强力，但需要搭配山吹的护盾（不被击中）以维持全伤加成☆活动圣痕，需等待活动复刻。通常利用2件套触发持续的点燃效果，以触发队友被动加伤或BOSS（黑轩辕）易伤。由于下位具备不错的辅助加成（20%火伤和物伤），因此通常是上、下或上、中（点燃）组合。☆活动圣痕，需等待活动复刻。上位的水球爆炸基本无伤害，核心在于中位的元素伤害提升效果。由于想触发中位必须携带上，所以通常使用上、中组合。作为活动圣痕，加成还是很不错的。相对叶采章，其加成稍低，且为延迟触发，不能立刻生效。地图装圣痕是各路神仙战场的常客，它们有的承担了极限伤害的补足、有的另辟蹊径实现了蛋池装备都不一定能实现的功能。好活。增幅游侠带姬麟黑套，月光上色效果怎样？。幽色倒是有了，没神恩，圣女得给黑希用，单月光和游侠有伤害吗？。游侠最好3s，可以用幽色猫黑黑，山吹带毕加索上色牛下，月光可以特毕奥武器索尔。👌🏻，我先肝到3s，还差几十片。好活。。现在还有萌新？。我呀，刚入坑。这个怎么肝啊，买不起委托书。。入坑。想求个发展思路。毕业雷律号入坑，打算抽丹心，那么炽翎还要继续养么，是不是要换樱桃炸弹。然后下一个养谁比较好（武器只有双狼）。我比较随意，不追求赶进度，但是不想走太多弯路不在意养成周期。。现在武器圣痕打造系统都看不懂，联机委托也不知道选啥。战场换粉毛，可以垃圾屋换血舞，攒水到周年庆，物理队的话神恩是必备的，炎八2s，山吹肝到增幅4星，金辉每周观星换完，星石买量力而行，不追求强度毕业雷律队够了红莲要多个体系配置完整，你有毕业雷律加个云墨血舞，初中级区红莲是没问题的。好的"
	print("test")
	print(len(b))
	a = " ".join(a)
	b = " ".join(b)
	res = rouge.get_scores(a, b)
	print(res)
	end_time = time.time()
	print("time:{}".format(end_time - start_time))

def test_file_process():
	from utils.file_process import excel_to_csv
	src_path = "/Users/supeng/Mirror/work_sync/data/operation/sentiment/miyoshe/长文本帖子收集.xlsx"
	tgt_path = "/private/tmp/tmp.csv"


	excel_to_csv(src_path, tgt_path, src_file_type="excel")


def test_ner():
	# from collections import defaultdict
	from result_analysis.ner_result import ner_classification_report
	# file_path = "/private/tmp/tmp/crf4j/test_data"
	# data = list()
	# with open(file_path, "r") as f:
	# 	cur_line = defaultdict(list)
	# 	for line in f.readlines():
	# 		if len(line.strip()) == 0 and cur_line:
	# 			data.append(cur_line)
	# 			cur_line = defaultdict(list)
	# 		line_list = line.strip().split("\t")
	# 		if len(line_list) == 3:
	# 			cur_line["text"].append(line_list[0])
	# 			cur_line["gold"].append(line_list[1])
	# 			cur_line["pred"].append(line_list[2])
	# print(data[:3])
	# golds = [x["gold"] for x in data]
	# preds = [x["pred"] for x in data]
	golds = [['O', 'O', 'O', 'B-MISC', 'I-MISC', 'I-MISC', 'O'], ['B-PER', 'I-PER', 'O']]
	preds = [['O', 'O', 'B-MISC', 'I-MISC', 'I-MISC', 'I-MISC', 'O'], ['B-PER', 'I-PER', 'O']]

	res = ner_classification_report(golds, preds, labels=["PER", "DA"])
	print(res)





if __name__ == '__main__':
	# test_file_process()
	test_ner()



#!user/bin/python
# _*_ coding: utf-8 _*_
# @Author      :   Jesper
# @Time        :   2021/7/20  10:38
# @Description :

from nlp_utils_ch.utils.logger import log


def read_table_data(file_path, col_names=None, header=False, sep="\t"):
	data = list()
	with open(file_path, "r") as f:
		count = 0
		for line in f.readlines():

			if count == 0 and header is True:
				col_names = line.strip().split(sep)
				count += 1
				continue

			line_list = line.strip().split(sep)
			assert len(col_names) == len(line_list)
			tmp_dict = dict()
			for i, col_name in enumerate(col_names):
				tmp_dict[col_name] = line_list[i]
			data.append(tmp_dict)
	log.info("data num:{}".format(len(data)))
	log.info("data example 2:{}".format(data[:2]))
	return data




# -*- coding: utf-8 -*-
import struct
import os

# 由于原代码不适用python3且有大量bug
# 以及有函数没有必要使用且一些代码书写不太规范或冗余
# 所以本人在原有的大框架基本不动的情况下作了大量的细节更改。
# 使得没有乱码出现，文件夹导入更方便等等。
# Author：Ling Yue, Taiyuan U of Tech
# Blog: http://blog.yueling.me


# 原作者：
# 搜狗的scel词库就是保存的文本的unicode编码，每两个字节一个字符（中文汉字或者英文字母）
# 找出其每部分的偏移位置即可
# 主要两部分
# 1.全局拼音表，貌似是所有的拼音组合，字典序
#       格式为(index,len,pinyin)的列表
#       index: 两个字节的整数 代表这个拼音的索引
#       len: 两个字节的整数 拼音的字节长度
#       pinyin: 当前的拼音，每个字符两个字节，总长len
#
# 2.汉语词组表
#       格式为(same,py_table_len,py_table,{word_len,word,ext_len,ext})的一个列表
#       same: 两个字节 整数 同音词数量
#       py_table_len:  两个字节 整数
#       py_table: 整数列表，每个整数两个字节,每个整数代表一个拼音的索引
#
#       word_len:两个字节 整数 代表中文词组字节数长度
#       word: 中文词组,每个中文汉字两个字节，总长度word_len
#       ext_len: 两个字节 整数 代表扩展信息的长度，好像都是10
#       ext: 扩展信息 前两个字节是一个整数(不知道是不是词频) 后八个字节全是0
#
#      {word_len,word,ext_len,ext} 一共重复same次 同音词 相同拼音表


# 拼音表偏移，
startPy = 0x1540;

# 汉语词组表偏移
startChinese = 0x2628;

# 全局拼音表
GPy_Table = {}

# 解析结果
# 元组(词频,拼音,中文词组)的列表
GTable = []

# 原始字节码转为字符串
def byte2str(data):
    pos = 0
    str = ''
    while pos < len(data):
        c = chr(struct.unpack('H', bytes([data[pos], data[pos + 1]]))[0])
        if c != chr(0):
            str += c
        pos += 2
    return str

# 获取拼音表
def getPyTable(data):
    data = data[4:]
    pos = 0
    while pos < len(data):
        index = struct.unpack('H', bytes([data[pos],data[pos + 1]]))[0]
        pos += 2
        lenPy = struct.unpack('H', bytes([data[pos], data[pos + 1]]))[0]
        pos += 2
        py = byte2str(data[pos:pos + lenPy])

        GPy_Table[index] = py
        pos += lenPy

# 获取一个词组的拼音
def getWordPy(data):
    pos = 0
    ret = ''
    while pos < len(data):
        index = struct.unpack('H', bytes([data[pos], data[pos + 1]]))[0]
        ret += GPy_Table[index]
        pos += 2
    return ret

# 读取中文表
def getChinese(data):
    pos = 0
    while pos < len(data):
        # 同音词数量
        same = struct.unpack('H', bytes([data[pos], data[pos + 1]]))[0]

        # 拼音索引表长度
        pos += 2
        py_table_len = struct.unpack('H', bytes([data[pos], data[pos + 1]]))[0]

        # 拼音索引表
        pos += 2
        py = getWordPy(data[pos: pos + py_table_len])

        # 中文词组
        pos += py_table_len
        for i in range(same):
            # 中文词组长度
            c_len = struct.unpack('H', bytes([data[pos], data[pos + 1]]))[0]
            # 中文词组
            pos += 2
            word = byte2str(data[pos: pos + c_len])
            # 扩展数据长度
            pos += c_len
            ext_len = struct.unpack('H', bytes([data[pos], data[pos + 1]]))[0]
            # 词频
            pos += 2
            count = struct.unpack('H', bytes([data[pos], data[pos + 1]]))[0]

            # 保存
            GTable.append((count, py, word))

            # 到下个词的偏移位置
            pos += ext_len


def scel2txt(file_name):
    print('-' * 60)
    with open(file_name, 'rb') as f:
        data = f.read()

    print("词库名：", byte2str(data[0x130:0x338])) # .encode('GB18030')
    print("词库类型：", byte2str(data[0x338:0x540]))
    print("描述信息：", byte2str(data[0x540:0xd40]))
    print("词库示例：", byte2str(data[0xd40:startPy]))

    getPyTable(data[startPy:startChinese])
    getChinese(data[startChinese:])

def convert_scel_to_txt(src_path_dir, target_file_path):
	# scel所在文件夹路径
	# 输出词典所在文件夹路径

	in_path = src_path_dir
	out_path = target_file_path

	fin = [fname for fname in os.listdir(in_path) if fname[-5:] == ".scel"]
	for f in fin:
		f = os.path.join(in_path, f)
		scel2txt(f)

	# 保存结果
	with open(out_path, 'w', encoding='utf8') as f:
		f.writelines([word + '\n' for count, py, word in GTable])


if __name__ == '__main__':
	pass

    # # scel所在文件夹路径
    # in_path = "/Users/really/Documents/coal_dict"
    # # 输出词典所在文件夹路径
    # out_path = "coal_dict.txt"
	#
    # fin = [fname for fname in os.listdir(in_path) if fname[-5:] == ".scel"]
    # for f in fin:
    #     f = os.path.join(in_path, f)
    #     scel2txt(f)
	#
    # # 保存结果
    # with open(out_path, 'w', encoding='utf8') as f:
    #     f.writelines([word+'\n' for count, py, word in GTable])



# -*- coding: utf-8 -*-
"""
   Author :        Jesper
   Date：          2020/2/25 09:52
   Description :
   Changed by:
"""

from collections import defaultdict

def classification_report(y_true, y_pred, decimal_place=4, use_f1=True,
                          use_confusion_matrix=False, print_report_str=True, show_label_num=True):
	assert len(y_true) == len(y_pred)

	report_str = "\n" + "="*20 + "[REPORT]" + "=" * 20 + "\n"

	report_str += total_accuracy(y_true, y_pred, decimal_place) + "\n"

	label_predict_detail, matrix_report_str = evaluation_metrics(y_true, y_pred, use_f1, use_confusion_matrix, decimal_place, show_label_num)

	report_str +=  matrix_report_str + "\n\n"
	for label, item in label_predict_detail.items():
		_label_report = item["report_str"]
		report_str += _label_report + "\n"


	if print_report_str:
		print(report_str)

	return report_str



def total_accuracy(y_true, y_pred, decimal_place=4):
	assert len(y_true) == len(y_pred)
	if_true_list = [1 if y_true[i] == y_pred[i] else 0 for i in range(len(y_true))]
	accuracy = sum(if_true_list) / len(y_true)
	accuracy = round(accuracy, decimal_place)
	acc_str = "Accuracy: {}, data num: {}, match num: {}".format(accuracy, len(y_true), sum(if_true_list))
	return acc_str





def evaluation_metrics(y_true, y_pred, use_f1=False, use_confusion_matrix=False, decimal_place=4, show_label_num=False):
	assert len(y_true) == len(y_pred)
	label_predict_detail = defaultdict(dict)
	labels_set = list(set(y_true))
	data_num = len(y_true)

	matrix_report_str = "| {:8}| {:6}| {:6}| {:6}| {:6}| {:6}| {:6}| ".format("label", "P", "R", "F1","TP" ,"TP+FP", "TP+FN")
	matrix_report_str += "\n" + "| ------- | ----- | ----- | ----- | ----- | ----- | ----- |"

	macro_p = 0
	macro_r = 0
	macro_f1 = 0
	for _label in labels_set:
		predict_p = sum([1 if y_pred[i] == _label else 0 for i in range(len(y_true))])
		true_p = sum([1 if y_true[i] == _label else 0 for i in range(len(y_true))])
		_tp_num = sum([1 if y_true[i] == _label and y_pred[i] == _label else 0 for i in range(len(y_true))])
		_tn_num = sum([1 if y_true[i] != _label and y_pred[i] != _label else 0 for i in range(len(y_true))])
		_fp_num = predict_p - _tp_num
		_fn_num = true_p - _tp_num

		precision = round(_tp_num * 1.0/predict_p, decimal_place) if predict_p != 0 else 0
		recall = round(_tp_num * 1.0/true_p, decimal_place) if true_p != 0 else 0
		f1 = round(2 * precision * recall / (precision + recall), decimal_place) if (precision + recall) != 0 else 0

		macro_p += precision
		macro_r += recall
		macro_f1 += f1

		matrix_report_str += "\n" + "| {:8}| {:6}| {:6}| {:6}| {:6}| {:6}| {:6}| ".format(_label, precision, recall, f1, _tp_num, predict_p, true_p)
		report_str = "Label: {:4}, Precision:{:6}, Recall:{:6}".format(_label, precision, recall)
		if use_f1:
			report_str += ",  f1:{:6}".format(f1)
		if show_label_num:
			report_str += ", TP:{:6}, pred_postive:{:6}, true_positive:{:6}".format(_tp_num, predict_p, true_p)
		if use_confusion_matrix:



			raise NotImplementedError



		label_predict_detail[_label]["TP"] = _tp_num
		label_predict_detail[_label]["FP"] = _fp_num
		label_predict_detail[_label]["FN"] = _fn_num
		label_predict_detail[_label]["TN"] = _tn_num
		label_predict_detail[_label]["F1"] = f1
		label_predict_detail[_label]["precision"] = precision
		label_predict_detail[_label]["recall"] = recall
		label_predict_detail[_label]["report_str"] = report_str

	macro_p = round(macro_p/len(labels_set), 4)
	macro_r = round(macro_r/len(labels_set), 4)
	macro_f1 = round(macro_f1/len(labels_set), 4)

	matrix_report_str += "\n" + "| {:8}| {:6}| {:6}| {:6}| {:6}| {:6}| {:6}| ".format("Macro", macro_p, macro_r, macro_f1,
	                                                                                  "", "", "")

	return label_predict_detail, matrix_report_str



#!user/bin/python
# _*_ coding: utf-8 _*_
# @Author      :   Jesper
# @Time        :   2021/9/27  16:13
# @Description :

from collections import Counter, defaultdict
from nlp_utils_ch.utils.logger import log

def suffix_most_common(words_list, suffix_len=2, exclude_count=3, show_topn_suffix=0):
    suffix_map = defaultdict(int)
    for _word in words_list:
        if len(_word) > suffix_len:
            suffix_word = _word[-suffix_len:]
            suffix_map[suffix_word] = suffix_map.get(suffix_word, 0) + 1

    suffix_map = filter(lambda x: x[1] >= exclude_count, suffix_map.items())
    suffix_map = dict(suffix_map)
    res_words = list()
    exclude_words = list()
    if show_topn_suffix > 0:
        to_show_suffix_map = sorted(suffix_map.items(), key=lambda kv: (kv[1], kv[0]), reverse=True)
        # to_show_suffix_map = [ v for v in sorted(suffix_map.values())]
        log.info("topn suffix:{}".format(to_show_suffix_map[:show_topn_suffix]))


    for _word in words_list:
        if len(_word) >= suffix_len:
            _suffix_word = _word[-suffix_len:]
            if _suffix_word not in suffix_map:
                res_words.append(_word)
            else:
                exclude_words.append(_word)

        else:
            res_words.append(_word)

    return res_words




if __name__ == '__main__':
    words = ["可爱见", "哈爱见", "爱见", "没左左"]
    res = suffix_most_common(words)
    # print(res)



#!user/bin/python
# _*_ coding: utf-8 _*_
# @Author      :   Jesper
# @Time        :   2020/10/29 15:05
# @Description :
import json
from nlp_utils_ch.utils.logger import log

def show_json(data, prefix=None, is_indent=False):
	show_str = ""
	if is_indent:
		show_str = json.dumps(data, ensure_ascii=False, indent=4)
	else:
		show_str = json.dumps(data, ensure_ascii=False)

	if prefix and isinstance(prefix, str):
		show_str = prefix + ":"  + show_str

	log.info(show_str)



#!user/bin/python
# _*_ coding: utf-8 _*_
# @Author      :   Jesper
# @Time        :   2020/11/30 11:09
# @Description :


import requests
import eventlet
import time

eventlet.monkey_patch()

time_limit = 3  # set timeout time 3s

with eventlet.Timeout(time_limit, False):
	while(True):
		time.sleep(1)
		print("time")
	# r = requests.get("https://me.csdn.net/dcrmg", verify=False)
	print('error')
print('over')



#!user/bin/python
# _*_ coding: utf-8 _*_
# @Author      :   Jesper
# @Time        :   2021/7/20  10:36
# @Description :
import hashlib

def get_md5(text, num=10):
	md5 = hashlib.md5()
	md5.update(text.encode('utf-8'))
	# print(md5.hexdigest())
	return md5.hexdigest()[:num]



#!user/bin/python
# _*_ coding: utf-8 _*_
# @Author      :   Jesper
# @Time        :   2021/9/7  11:18
# @Description :


def remove_duplicate_keep_order(lines):
	res_line = list()
	res_set = set()
	for line in lines:
		if line not in res_set:
			res_line.append(line)
			res_set.add(line)
	return res_line


def unique_query(raw_list):
	res_list = list()
	for item in raw_list:
		if item not in res_list:
			res_list.append(item)
	return res_list



#!user/bin/python
# _*_ coding: utf-8 _*_
# @Author      :   Jesper
# @Time        :   2020/12/22 10:11
# @Description :
import re

def filter_redundant_char(text):
    text = text.lower()
    # 去除网址
    text = re.sub(r"(https|http)?:\/\/(\w|\.|\/|\?|\=|\&|\%)*\b", "", text)
    # 合并正文中过多的空格
    text = re.sub(r"\s+", " ", text)
    # 去除网页符号
    html_re = re.compile(r'<[^>]+>', re.S)
    text = html_re.sub('', text)

    uid_re = re.compile(r'^\d{n}', re.S)
    # text = uid_re.sub('', text)
    text = re.sub('uid[:：][0-9]+', '', text)
    return text


if __name__ == '__main__':
    text = "{'describe':'画渣一个','imgs':['https://upload-bbs.mihoyo.com/upload/2020/07/08/100898502/fa2b5ff511541be3270d9df2f6ca65fa_2660021923020391956.jpg','https://upload-bbs.mihoyo.com/upload/2020/07/08/100898502/bac9952afd28723ae68ec368a9739d86_2899968460626170607.jpg']}"
    res = filter_redundant_char(text)
    print(res)



#!user/bin/python
# _*_ coding: utf-8 _*_
# @Author      :   Jesper
# @Time        :   2022/11/11  10:45
# @Description :




def delete_tab_line_break(text):
	text = str(text).replace("\n", " ").replace("\r", " ").replace("\t", " ")
	return text



# -*- coding: utf-8 -*-
"""
   Author :        Jesper
   Date：          2020/2/25 09:51
   Description :
   Changed by:
"""
import numpy as np
from collections import defaultdict


def get_entity_detail(cur_text, predict_labels):
	pred_entities_info = get_entities(predict_labels)
	entities_words = list()
	for _pred_entities_info in pred_entities_info:
		_label = _pred_entities_info[0]
		_pos_start = _pred_entities_info[1]
		_pos_end = _pred_entities_info[2]
		entities_words.append({
			"word": cur_text[_pos_start:_pos_end + 1],
			"label": _label,
			"pos": [_pos_start, _pos_end + 1]

		})
	return entities_words


def get_entities(seq, suffix=False):
	"""Gets entities from sequence.

	Args:
		seq (list): sequence of labels.

	Returns:
		list: list of (chunk_type, chunk_start, chunk_end).

	Example:
		>>> from seqeval.metrics.sequence_labeling import get_entities
		>>> seq = ['B-PER', 'I-PER', 'O', 'B-LOC']
		>>> get_entities(seq)
		[('PER', 0, 1), ('LOC', 3, 3)]
	"""
	# for nested list
	if any(isinstance(s, list) for s in seq):
		seq = [item for sublist in seq for item in sublist + ['O']]

	prev_tag = 'O'
	prev_type = ''
	begin_offset = 0
	chunks = []
	for i, chunk in enumerate(seq + ['O']):
		if suffix:
			tag = chunk[-1]
			type_ = chunk.split('-')[0]
		else:
			tag = chunk[0]
			type_ = chunk.split('-')[-1]

		if end_of_chunk(prev_tag, tag, prev_type, type_):
			chunks.append((prev_type, begin_offset, i-1))
		if start_of_chunk(prev_tag, tag, prev_type, type_):
			begin_offset = i
		prev_tag = tag
		prev_type = type_

	return chunks


def end_of_chunk(prev_tag, tag, prev_type, type_):
	"""Checks if a chunk ended between the previous and current word.

	Args:
		prev_tag: previous chunk tag.
		tag: current chunk tag.
		prev_type: previous type.
		type_: current type.

	Returns:
		chunk_end: boolean.
	"""
	chunk_end = False

	if prev_tag == 'E': chunk_end = True
	if prev_tag == 'S': chunk_end = True

	if prev_tag == 'B' and tag == 'B': chunk_end = True
	if prev_tag == 'B' and tag == 'S': chunk_end = True
	if prev_tag == 'B' and tag == 'O': chunk_end = True
	if prev_tag == 'I' and tag == 'B': chunk_end = True
	if prev_tag == 'I' and tag == 'S': chunk_end = True
	if prev_tag == 'I' and tag == 'O': chunk_end = True

	if prev_tag != 'O' and prev_tag != '.' and prev_type != type_:
		chunk_end = True

	return chunk_end


def start_of_chunk(prev_tag, tag, prev_type, type_):
	"""Checks if a chunk started between the previous and current word.

	Args:
		prev_tag: previous chunk tag.
		tag: current chunk tag.
		prev_type: previous type.
		type_: current type.

	Returns:
		chunk_start: boolean.
	"""
	chunk_start = False

	if tag == 'B': chunk_start = True
	if tag == 'S': chunk_start = True

	if prev_tag == 'E' and tag == 'E': chunk_start = True
	if prev_tag == 'E' and tag == 'I': chunk_start = True
	if prev_tag == 'S' and tag == 'E': chunk_start = True
	if prev_tag == 'S' and tag == 'I': chunk_start = True
	if prev_tag == 'O' and tag == 'E': chunk_start = True
	if prev_tag == 'O' and tag == 'I': chunk_start = True

	if tag != 'O' and tag != '.' and prev_type != type_:
		chunk_start = True

	return chunk_start


def f1_score(y_true, y_pred, average='micro', suffix=False):
	"""Compute the F1 score.

	The F1 score can be interpreted as a weighted average of the precision and
	recall, where an F1 score reaches its best value at 1 and worst score at 0.
	The relative contribution of precision and recall to the F1 score are
	equal. The formula for the F1 score is::

		F1 = 2 * (precision * recall) / (precision + recall)

	Args:
		y_true : 2d array. Ground truth (correct) target values.
		y_pred : 2d array. Estimated targets as returned by a tagger.

	Returns:
		score : float.

	Example:
		>>> from seqeval.metrics import f1_score
		>>> y_true = [['O', 'O', 'O', 'B-MISC', 'I-MISC', 'I-MISC', 'O'], ['B-PER', 'I-PER', 'O']]
		>>> y_pred = [['O', 'O', 'B-MISC', 'I-MISC', 'I-MISC', 'I-MISC', 'O'], ['B-PER', 'I-PER', 'O']]
		>>> f1_score(y_true, y_pred)
		0.50
	"""
	true_entities = set(get_entities(y_true, suffix))
	pred_entities = set(get_entities(y_pred, suffix))

	nb_correct = len(true_entities & pred_entities)
	nb_pred = len(pred_entities)
	nb_true = len(true_entities)

	p = nb_correct / nb_pred if nb_pred > 0 else 0
	r = nb_correct / nb_true if nb_true > 0 else 0
	score = 2 * p * r / (p + r) if p + r > 0 else 0

	return score


def accuracy_score(y_true, y_pred):
	"""Accuracy classification score.

	In multilabel classification, this function computes subset accuracy:
	the set of labels predicted for a sample must *exactly* match the
	corresponding set of labels in y_true.

	Args:
		y_true : 2d array. Ground truth (correct) target values.
		y_pred : 2d array. Estimated targets as returned by a tagger.

	Returns:
		score : float.

	Example:
		>>> from seqeval.metrics import accuracy_score
		>>> y_true = [['O', 'O', 'O', 'B-MISC', 'I-MISC', 'I-MISC', 'O'], ['B-PER', 'I-PER', 'O']]
		>>> y_pred = [['O', 'O', 'B-MISC', 'I-MISC', 'I-MISC', 'I-MISC', 'O'], ['B-PER', 'I-PER', 'O']]
		>>> accuracy_score(y_true, y_pred)
		0.80
	"""
	if any(isinstance(s, list) for s in y_true):
		y_true = [item for sublist in y_true for item in sublist]
		y_pred = [item for sublist in y_pred for item in sublist]

	nb_correct = sum(y_t==y_p for y_t, y_p in zip(y_true, y_pred))
	nb_true = len(y_true)

	score = nb_correct / nb_true

	return score


def precision_score(y_true, y_pred, average='micro', suffix=False):
	"""Compute the precision.

	The precision is the ratio ``tp / (tp + fp)`` where ``tp`` is the number of
	true positives and ``fp`` the number of false positives. The precision is
	intuitively the ability of the classifier not to label as positive a sample.

	The best value is 1 and the worst value is 0.

	Args:
		y_true : 2d array. Ground truth (correct) target values.
		y_pred : 2d array. Estimated targets as returned by a tagger.

	Returns:
		score : float.

	Example:
		>>> from seqeval.metrics import precision_score
		>>> y_true = [['O', 'O', 'O', 'B-MISC', 'I-MISC', 'I-MISC', 'O'], ['B-PER', 'I-PER', 'O']]
		>>> y_pred = [['O', 'O', 'B-MISC', 'I-MISC', 'I-MISC', 'I-MISC', 'O'], ['B-PER', 'I-PER', 'O']]
		>>> precision_score(y_true, y_pred)
		0.50
	"""
	true_entities = set(get_entities(y_true, suffix))
	pred_entities = set(get_entities(y_pred, suffix))

	nb_correct = len(true_entities & pred_entities)
	nb_pred = len(pred_entities)

	score = nb_correct / nb_pred if nb_pred > 0 else 0

	return score


def recall_score(y_true, y_pred, average='micro', suffix=False):
	"""Compute the recall.

	The recall is the ratio ``tp / (tp + fn)`` where ``tp`` is the number of
	true positives and ``fn`` the number of false negatives. The recall is
	intuitively the ability of the classifier to find all the positive samples.

	The best value is 1 and the worst value is 0.

	Args:
		y_true : 2d array. Ground truth (correct) target values.
		y_pred : 2d array. Estimated targets as returned by a tagger.

	Returns:
		score : float.

	Example:
		>>> from seqeval.metrics import recall_score
		>>> y_true = [['O', 'O', 'O', 'B-MISC', 'I-MISC', 'I-MISC', 'O'], ['B-PER', 'I-PER', 'O']]
		>>> y_pred = [['O', 'O', 'B-MISC', 'I-MISC', 'I-MISC', 'I-MISC', 'O'], ['B-PER', 'I-PER', 'O']]
		>>> recall_score(y_true, y_pred)
		0.50
	"""
	true_entities = set(get_entities(y_true, suffix))
	pred_entities = set(get_entities(y_pred, suffix))

	nb_correct = len(true_entities & pred_entities)
	nb_true = len(true_entities)

	score = nb_correct / nb_true if nb_true > 0 else 0

	return score


def performance_measure(y_true, y_pred):
	"""
	Compute the performance metrics: TP, FP, FN, TN

	Args:
		y_true : 2d array. Ground truth (correct) target values.
		y_pred : 2d array. Estimated targets as returned by a tagger.

	Returns:
		performance_dict : dict

	Example:
		>>> from seqeval.metrics import performance_measure
		>>> y_true = [['O', 'O', 'O', 'B-MISC', 'I-MISC', 'O', 'B-ORG'], ['B-PER', 'I-PER', 'O']]
		>>> y_pred = [['O', 'O', 'B-MISC', 'I-MISC', 'I-MISC', 'O', 'O'], ['B-PER', 'I-PER', 'O']]
		>>> performance_measure(y_true, y_pred)
		(3, 3, 1, 4)
	"""
	performace_dict = dict()
	if any(isinstance(s, list) for s in y_true):
		y_true = [item for sublist in y_true for item in sublist]
		y_pred = [item for sublist in y_pred for item in sublist]
	performace_dict['TP'] = sum(y_t == y_p for y_t, y_p in zip(y_true, y_pred)
								if ((y_t != 'O') or (y_p != 'O')))
	performace_dict['FP'] = sum(y_t != y_p for y_t, y_p in zip(y_true, y_pred))
	performace_dict['FN'] = sum(((y_t != 'O') and (y_p == 'O'))
								for y_t, y_p in zip(y_true, y_pred))
	performace_dict['TN'] = sum((y_t == y_p == 'O')
								for y_t, y_p in zip(y_true, y_pred))

	return performace_dict


def ner_classification_report(y_true, y_pred, digits=4, suffix=False, labels=None):
	"""Build a text report showing the main classification metrics.

	Args:
		y_true : 2d array. Ground truth (correct) target values.
		y_pred : 2d array. Estimated targets as returned by a classifier.
		digits : int. Number of digits for formatting output floating point values.

	Returns:
		report : string. Text summary of the precision, recall, F1 score for each class.

	Examples:
		>>> from nlp_utils_ch.result_analysis.ner_result import ner_classification_report
		>>> y_true = [['O', 'O', 'O', 'B-MISC', 'I-MISC', 'I-MISC', 'O'], ['B-PER', 'I-PER', 'O']]
		>>> y_pred = [['O', 'O', 'B-MISC', 'I-MISC', 'I-MISC', 'I-MISC', 'O'], ['B-PER', 'I-PER', 'O']]
		>>> print(ner_classification_report(y_true, y_pred))
					 precision    recall  f1-score   support
		<BLANKLINE>
			   MISC       0.00      0.00      0.00         1
				PER       1.00      1.00      1.00         1
		<BLANKLINE>
		  micro avg       0.50      0.50      0.50         2
		  macro avg       0.50      0.50      0.50         2
		<BLANKLINE>
	"""
	true_entities = set(get_entities(y_true, suffix))
	pred_entities = set(get_entities(y_pred, suffix))

	name_width = 0
	d1 = defaultdict(set)
	d2 = defaultdict(set)
	for e in true_entities:
		d1[e[0]].add((e[1], e[2]))
		name_width = max(name_width, len(e[0]))
	for e in pred_entities:
		d2[e[0]].add((e[1], e[2]))

	last_line_heading = 'macro avg'
	width = max(name_width, len(last_line_heading), digits)

	headers = ["precision", "recall", "f1-score", "support"]
	head_fmt = u'|{:>{width}s}| ' + u' {:>9}|' * (len(headers) - 1) + u' {:>9}|'
	report = head_fmt.format(u'Label', *headers, width=width)
	report += u'\n'
	report += "| ------- | --------- | -------- | -------- | -------- |\n"

	row_fmt = u'|{:>{width}s}| ' + u' {:>9.{digits}f}|' * 3 + u' {:>9}|\n'

	ps, rs, f1s, s = [], [], [], []
	# 基于指定label顺序输出结果
	report_labels = list()
	if labels is not None:
		for _label in labels:
			if _label not in report_labels:
				report_labels.append(_label)
	d1_labels = d1.keys()
	for _label in d1_labels:
		if _label not in report_labels:
			report_labels.append(_label)

	# for type_name, true_entities in d1.items():
	for _label in report_labels:
		if _label not in d1:
			print("label:{} error".format(_label))
			continue
		type_name = _label
		true_entities = d1[_label]
		pred_entities = d2[type_name]
		nb_correct = len(true_entities & pred_entities)
		nb_pred = len(pred_entities)
		nb_true = len(true_entities)

		p = nb_correct / nb_pred if nb_pred > 0 else 0
		r = nb_correct / nb_true if nb_true > 0 else 0
		f1 = 2 * p * r / (p + r) if p + r > 0 else 0

		report += row_fmt.format(*[type_name, p, r, f1, nb_true], width=width, digits=digits)

		ps.append(p)
		rs.append(r)
		f1s.append(f1)
		s.append(nb_true)

	# report += u'\n'

	# compute averages
	report += row_fmt.format('micro avg',
							 precision_score(y_true, y_pred, suffix=suffix),
							 recall_score(y_true, y_pred, suffix=suffix),
							 f1_score(y_true, y_pred, suffix=suffix),
							 np.sum(s),
							 width=width, digits=digits)
	report += row_fmt.format(last_line_heading,
							 np.average(ps, weights=s),
							 np.average(rs, weights=s),
							 np.average(f1s, weights=s),
							 np.sum(s),
							 width=width, digits=digits)

	return report



# -*- coding: utf-8 -*-
"""
   Author :        Jesper
   Date：          2020/3/3 15:24
   Description :
   Changed by:
"""

import re

# 去除括号内容，避免对模型造成干扰
# todo: badcase  挂精卫(哪个位置？_(非)常疑惑)

def filter_text_between_bracket(text):
	l_symbols = ["(", "）", "[", "【"]
	r_symbols = [")", "）", "]", "】"]
	bracket_positions = list()
	text_len = len(text)
	i = 0
	while i < text_len:
		if text[i] in l_symbols:
			j = i+1
			while j <  text_len and text[j] not in r_symbols:
				j += 1
			if j < text_len and text[j] in r_symbols and l_symbols.index(text[i]) == r_symbols.index(text[j]):
				bracket_positions.append([i,j])
				i = j
		i += 1
	return remove_text_by_section(text, bracket_positions)


def bracket_words_process(text, bracket_symbols=None, is_filter=False):
	if bracket_symbols is None:
		l_symbols = ["(", "）", "[", "【", "「"]
		r_symbols = [")", "）", "]", "】", "」"]
	else:
		l_symbols = [x[0] for x in bracket_symbols]
		r_symbols = [x[1] for x in bracket_symbols]
	bracket_positions = list()
	text_len = len(text)
	i = 0
	while i < text_len:
		if text[i] in l_symbols:
			j = i + 1
			while j < text_len and text[j] not in r_symbols:
				j += 1
			if j < text_len and text[j] in r_symbols and l_symbols.index(text[i]) == r_symbols.index(text[j]):
				bracket_positions.append([i, j])
				i = j
		i += 1
	if is_filter is True:
		return remove_text_by_section(text, bracket_positions)
	else:
		words_list = list()
		for _position in bracket_positions:
			# print(_position)
			_word = text[_position[0]+1:_position[1]]
			if _word:
				words_list.append(_word)

		return words_list





def remove_text_by_section(text, sections):
	"""
	:param text:
	:param sections: e.g. [[10, 15], [18, 23]]
	:return:
	"""
	res_text = ""
	start_index = 0
	for _section in sections:
		res_text += text[start_index:_section[0]]
		start_index = _section[1] + 1
	res_text += text[start_index:len(text)]
	return res_text


# 删除网页标签
def delete_html_tag(s):
	s = re.sub('\{IMG:.?.?.?\}', '', s)  # 图片
	s = re.sub(re.compile(r'[a-zA-Z]+://[^\s]+'), '', s)  # 网址
	s = re.sub(re.compile('<.*?>'), '', s)  # 网页标签
	s = re.sub(re.compile('&[a-zA-Z]+;?'), ' ', s)  # 网页标签
	s = re.sub(re.compile('[a-zA-Z0-9]*[./]+[a-zA-Z0-9./]+[a-zA-Z0-9./]*'), ' ', s)
	s = re.sub("\?{2,}", "", s)
	s = re.sub("\r", "", s)
	s = re.sub("\n", ",", s)
	s = re.sub("\t", ",", s)
	s = re.sub("（", ",", s)
	s = re.sub("）", ",", s)
	s = re.sub("\u3000", "", s)
	s = re.sub(" ", "", s)
	r4 = re.compile('\d{4}[-/]\d{2}[-/]\d{2}')  # 日期
	return s

def merege_seq_duplicated_to_one(text, duplicated_char):
	splited_text = text.split(duplicated_char)
	res_text = list()
	for _text in splited_text:
		if _text:
			res_text.append(_text)
	res_text = duplicated_char.join(res_text)
	return res_text










import os
import string
import io
"""
   Author :        Jesper
   Date：          2020/2/24 20:49
   Description :
   Changed by:
"""


class KeywordProcessor(object):
    """KeywordProcessor

    Attributes:
        _keyword (str): Used as key to store keywords in trie dictionary.
            Defaults to '_keyword_'
        non_word_boundaries (set(str)): Characters that will determine if the word is continuing.
            Defaults to set([A-Za-z0-9_])
        keyword_trie_dict (dict): Trie dict built character by character, that is used for lookup
            Defaults to empty dictionary
        case_sensitive (boolean): if the search algorithm should be case sensitive or not.
            Defaults to False

    Examples:
        >>> # import module
        >>> from flashtext import KeywordProcessor
        >>> # Create an object of KeywordProcessor
        >>> keyword_processor = KeywordProcessor()
        >>> # add keywords
        >>> keyword_names = ['NY', 'new-york', 'SF']
        >>> clean_names = ['new york', 'new york', 'san francisco']
        >>> for keyword_name, clean_name in zip(keyword_names, clean_names):
        >>>     keyword_processor.add_keyword(keyword_name, clean_name)
        >>> keywords_found = keyword_processor.extract_keywords('I love SF and NY. new-york is the best.')
        >>> keywords_found
        >>> ['san francisco', 'new york', 'new york']

    Note:
        * loosely based on `Aho-Corasick algorithm <https://en.wikipedia.org/wiki/Aho%E2%80%93Corasick_algorithm>`_.
        * Idea came from this `Stack Overflow Question <https://stackoverflow.com/questions/44178449/regex-replace-is-taking-time-for-millions-of-documents-how-to-make-it-faster>`_.
    """

    def __init__(self, case_sensitive=False):
        """
        Args:
            case_sensitive (boolean): Keyword search should be case sensitive set or not.
                Defaults to False
        """
        self._keyword = '_keyword_'
        self._white_space_chars = set(['.', '\t', '\n', '\a', ' ', ','])
        try:
            # python 2.x
            # self.non_word_boundaries = set(string.digits + string.letters + '_')
            self.non_word_boundaries = set('_')
        except AttributeError:
            # python 3.x
            # self.non_word_boundaries = set(string.digits + string.ascii_letters + '_')
            self.non_word_boundaries = set('_')
        self.keyword_trie_dict = dict()
        self.case_sensitive = case_sensitive
        self._terms_in_trie = 0

    def __len__(self):
        """Number of terms present in the keyword_trie_dict

        Returns:
            length : int
                Count of number of distinct terms in trie dictionary.

        """
        return self._terms_in_trie

    def __contains__(self, word):
        """To check if word is present in the keyword_trie_dict

        Args:
            word : string
                word that you want to check

        Returns:
            status : bool
                If word is present as it is in keyword_trie_dict then we return True, else False

        Examples:
            >>> keyword_processor.add_keyword('Big Apple')
            >>> 'Big Apple' in keyword_processor
            >>> # True

        """
        if not self.case_sensitive:
            word = word.lower()
        current_dict = self.keyword_trie_dict
        len_covered = 0
        for char in word:
            if char in current_dict:
                current_dict = current_dict[char]
                len_covered += 1
            else:
                break
        return self._keyword in current_dict and len_covered == len(word)

    def __getitem__(self, word):
        """if word is present in keyword_trie_dict return the clean name for it.

        Args:
            word : string
                word that you want to check

        Returns:
            keyword : string
                If word is present as it is in keyword_trie_dict then we return keyword mapped to it.

        Examples:
            >>> keyword_processor.add_keyword('Big Apple', 'New York')
            >>> keyword_processor['Big Apple']
            >>> # New York
        """
        if not self.case_sensitive:
            word = word.lower()
        current_dict = self.keyword_trie_dict
        len_covered = 0
        for char in word:
            if char in current_dict:
                current_dict = current_dict[char]
                len_covered += 1
            else:
                break
        if self._keyword in current_dict and len_covered == len(word):
            return current_dict[self._keyword]

    def __setitem__(self, keyword, clean_name=None):
        """To add keyword to the dictionary
        pass the keyword and the clean name it maps to.

        Args:
            keyword : string
                keyword that you want to identify

            clean_name : string
                clean term for that keyword that you would want to get back in return or replace
                if not provided, keyword will be used as the clean name also.

        Examples:
            >>> keyword_processor['Big Apple'] = 'New York'
        """
        status = False
        if not clean_name and keyword:
            clean_name = keyword

        if keyword and clean_name:
            if not self.case_sensitive:
                keyword = keyword.lower()
            current_dict = self.keyword_trie_dict
            for letter in keyword:
                current_dict = current_dict.setdefault(letter, {})
            if self._keyword not in current_dict:
                status = True
                self._terms_in_trie += 1
            current_dict[self._keyword] = clean_name
        return status

    def __delitem__(self, keyword):
        """To remove keyword from the dictionary
        pass the keyword and the clean name it maps to.

        Args:
            keyword : string
                keyword that you want to remove if it's present

        Examples:
            >>> keyword_processor.add_keyword('Big Apple')
            >>> del keyword_processor['Big Apple']
        """
        status = False
        if keyword:
            if not self.case_sensitive:
                keyword = keyword.lower()
            current_dict = self.keyword_trie_dict
            character_trie_list = []
            for letter in keyword:
                if letter in current_dict:
                    character_trie_list.append((letter, current_dict))
                    current_dict = current_dict[letter]
                else:
                    # if character is not found, break out of the loop
                    current_dict = None
                    break
            # remove the characters from trie dict if there are no other keywords with them
            if current_dict and self._keyword in current_dict:
                # we found a complete match for input keyword.
                character_trie_list.append((self._keyword, current_dict))
                character_trie_list.reverse()

                for key_to_remove, dict_pointer in character_trie_list:
                    if len(dict_pointer.keys()) == 1:
                        dict_pointer.pop(key_to_remove)
                    else:
                        # more than one key means more than 1 path.
                        # Delete not required path and keep the other
                        dict_pointer.pop(key_to_remove)
                        break
                # successfully removed keyword
                status = True
                self._terms_in_trie -= 1
        return status

    def __iter__(self):
        """Disabled iteration as get_all_keywords() is the right way to iterate
        """
        raise NotImplementedError("Please use get_all_keywords() instead")

    def set_non_word_boundaries(self, non_word_boundaries):
        """set of characters that will be considered as part of word.

        Args:
            non_word_boundaries (set(str)):
                Set of characters that will be considered as part of word.

        """
        self.non_word_boundaries = non_word_boundaries

    def add_non_word_boundary(self, character):
        """add a character that will be considered as part of word.

        Args:
            character (char):
                Character that will be considered as part of word.

        """
        self.non_word_boundaries.add(character)

    def add_keyword(self, keyword, clean_name=None):
        """To add one or more keywords to the dictionary
        pass the keyword and the clean name it maps to.

        Args:
            keyword : string
                keyword that you want to identify

            clean_name : string
                clean term for that keyword that you would want to get back in return or replace
                if not provided, keyword will be used as the clean name also.

        Returns:
            status : bool
                The return value. True for success, False otherwise.

        Examples:
            >>> keyword_processor.add_keyword('Big Apple', 'New York')
            >>> # This case 'Big Apple' will return 'New York'
            >>> # OR
            >>> keyword_processor.add_keyword('Big Apple')
            >>> # This case 'Big Apple' will return 'Big Apple'
        """
        return self.__setitem__(keyword, clean_name)

    def remove_keyword(self, keyword):
        """To remove one or more keywords from the dictionary
        pass the keyword and the clean name it maps to.

        Args:
            keyword : string
                keyword that you want to remove if it's present

        Returns:
            status : bool
                The return value. True for success, False otherwise.

        Examples:
            >>> keyword_processor.add_keyword('Big Apple')
            >>> keyword_processor.remove_keyword('Big Apple')
            >>> # Returns True
            >>> # This case 'Big Apple' will no longer be a recognized keyword
            >>> keyword_processor.remove_keyword('Big Apple')
            >>> # Returns False

        """
        return self.__delitem__(keyword)

    def get_keyword(self, word):
        """if word is present in keyword_trie_dict return the clean name for it.

        Args:
            word : string
                word that you want to check

        Returns:
            keyword : string
                If word is present as it is in keyword_trie_dict then we return keyword mapped to it.

        Examples:
            >>> keyword_processor.add_keyword('Big Apple', 'New York')
            >>> keyword_processor.get('Big Apple')
            >>> # New York
        """
        return self.__getitem__(word)

    def add_keyword_from_file(self, keyword_file, encoding="utf-8"):
        """To add keywords from a file

        Args:
            keyword_file : path to keywords file
            encoding : specify the encoding of the file

        Examples:
            keywords file format can be like:

            >>> # Option 1: keywords.txt content
            >>> # java_2e=>java
            >>> # java programing=>java
            >>> # product management=>product management
            >>> # product management techniques=>product management

            >>> # Option 2: keywords.txt content
            >>> # java
            >>> # python
            >>> # c++

            >>> keyword_processor.add_keyword_from_file('keywords.txt')

        Raises:
            IOError: If `keyword_file` path is not valid

        """
        if not os.path.isfile(keyword_file):
            raise IOError("Invalid file path {}".format(keyword_file))
        with io.open(keyword_file, encoding=encoding) as f:
            for line in f:
                if '=>' in line:
                    keyword, clean_name = line.split('=>')
                    self.add_keyword(keyword, clean_name.strip())
                else:
                    keyword = line.strip()
                    self.add_keyword(keyword)

    def add_keywords_from_dict(self, keyword_dict):
        """To add keywords from a dictionary

        Args:
            keyword_dict (dict): A dictionary with `str` key and (list `str`) as value

        Examples:
            >>> keyword_dict = {
                    "java": ["java_2e", "java programing"],
                    "product management": ["PM", "product manager"]
                }
            >>> keyword_processor.add_keywords_from_dict(keyword_dict)

        Raises:
            AttributeError: If value for a key in `keyword_dict` is not a list.

        """
        for clean_name, keywords in keyword_dict.items():
            if not isinstance(keywords, list):
                raise AttributeError("Value of key {} should be a list".format(clean_name))

            for keyword in keywords:
                self.add_keyword(keyword, clean_name)

    def remove_keywords_from_dict(self, keyword_dict):
        """To remove keywords from a dictionary

        Args:
            keyword_dict (dict): A dictionary with `str` key and (list `str`) as value

        Examples:
            >>> keyword_dict = {
                    "java": ["java_2e", "java programing"],
                    "product management": ["PM", "product manager"]
                }
            >>> keyword_processor.remove_keywords_from_dict(keyword_dict)

        Raises:
            AttributeError: If value for a key in `keyword_dict` is not a list.

        """
        for clean_name, keywords in keyword_dict.items():
            if not isinstance(keywords, list):
                raise AttributeError("Value of key {} should be a list".format(clean_name))

            for keyword in keywords:
                self.remove_keyword(keyword)

    def add_keywords_from_list(self, keyword_list):
        """To add keywords from a list

        Args:
            keyword_list (list(str)): List of keywords to add

        Examples:
            >>> keyword_processor.add_keywords_from_list(["java", "python"]})
        Raises:
            AttributeError: If `keyword_list` is not a list.

        """
        if not isinstance(keyword_list, list):
            raise AttributeError("keyword_list should be a list")

        for keyword in keyword_list:
            self.add_keyword(keyword)

    def remove_keywords_from_list(self, keyword_list):
        """To remove keywords present in list

        Args:
            keyword_list (list(str)): List of keywords to remove

        Examples:
            >>> keyword_processor.remove_keywords_from_list(["java", "python"]})
        Raises:
            AttributeError: If `keyword_list` is not a list.

        """
        if not isinstance(keyword_list, list):
                raise AttributeError("keyword_list should be a list")

        for keyword in keyword_list:
            self.remove_keyword(keyword)

    def get_all_keywords(self, term_so_far='', current_dict=None):
        """Recursively builds a dictionary of keywords present in the dictionary
        And the clean name mapped to those keywords.

        Args:
            term_so_far : string
                term built so far by adding all previous characters
            current_dict : dict
                current recursive position in dictionary

        Returns:
            terms_present : dict
                A map of key and value where each key is a term in the keyword_trie_dict.
                And value mapped to it is the clean name mapped to it.

        Examples:
            >>> keyword_processor = KeywordProcessor()
            >>> keyword_processor.add_keyword('j2ee', 'Java')
            >>> keyword_processor.add_keyword('Python', 'Python')
            >>> keyword_processor.get_all_keywords()
            >>> {'j2ee': 'Java', 'python': 'Python'}
            >>> # NOTE: for case_insensitive all keys will be lowercased.
        """
        terms_present = {}
        if not term_so_far:
            term_so_far = ''
        if current_dict is None:
            current_dict = self.keyword_trie_dict
        for key in current_dict:
            if key == '_keyword_':
                terms_present[term_so_far] = current_dict[key]
            else:
                sub_values = self.get_all_keywords(term_so_far + key, current_dict[key])
                for key in sub_values:
                    terms_present[key] = sub_values[key]
        return terms_present

    def extract_keywords(self, sentence, span_info=False):
        """Searches in the string for all keywords present in corpus.
        Keywords present are added to a list `keywords_extracted` and returned.

        Args:
            sentence (str): Line of text where we will search for keywords

        Returns:
            keywords_extracted (list(str)): List of terms/keywords found in sentence that match our corpus

        Examples:
            >>> from flashtext import KeywordProcessor
            >>> keyword_processor = KeywordProcessor()
            >>> keyword_processor.add_keyword('Big Apple', 'New York')
            >>> keyword_processor.add_keyword('Bay Area')
            >>> keywords_found = keyword_processor.extract_keywords('I love Big Apple and Bay Area.')
            >>> keywords_found
            >>> ['New York', 'Bay Area']

        """
        keywords_extracted = []
        if not sentence:
            # if sentence is empty or none just return empty list
            return keywords_extracted
        if not self.case_sensitive:
            sentence = sentence.lower()
        current_dict = self.keyword_trie_dict
        sequence_start_pos = 0
        sequence_end_pos = 0
        reset_current_dict = False
        idx = 0
        sentence_len = len(sentence)
        while idx < sentence_len:
            char = sentence[idx]
            # when we reach a character that might denote word end
            if char not in self.non_word_boundaries:

                # if end is present in current_dict
                if self._keyword in current_dict or char in current_dict:
                    # update longest sequence found
                    sequence_found = None
                    longest_sequence_found = None
                    is_longer_seq_found = False
                    if self._keyword in current_dict:
                        sequence_found = current_dict[self._keyword]
                        longest_sequence_found = current_dict[self._keyword]
                        sequence_end_pos = idx

                    # re look for longest_sequence from this position
                    if char in current_dict:
                        current_dict_continued = current_dict[char]

                        idy = idx + 1
                        while idy < sentence_len:
                            inner_char = sentence[idy]
                            if inner_char not in self.non_word_boundaries and self._keyword in current_dict_continued:
                                # update longest sequence found
                                longest_sequence_found = current_dict_continued[self._keyword]
                                sequence_end_pos = idy
                                is_longer_seq_found = True
                            if inner_char in current_dict_continued:
                                current_dict_continued = current_dict_continued[inner_char]
                            else:
                                break
                            idy += 1
                        else:
                            # end of sentence reached.
                            if self._keyword in current_dict_continued:
                                # update longest sequence found
                                longest_sequence_found = current_dict_continued[self._keyword]
                                sequence_end_pos = idy
                                is_longer_seq_found = True
                        if is_longer_seq_found:
                            idx = sequence_end_pos
                    current_dict = self.keyword_trie_dict
                    if longest_sequence_found:
                        keywords_extracted.append((longest_sequence_found, sequence_start_pos, idx))
                        # 20180905 add
                        idx = idx - 1
                    reset_current_dict = True
                else:
                    # we reset current_dict
                    current_dict = self.keyword_trie_dict
                    reset_current_dict = True
            elif char in current_dict:
                # we can continue from this char
                current_dict = current_dict[char]
            else:
                # we reset current_dict
                current_dict = self.keyword_trie_dict
                reset_current_dict = True
                # skip to end of word
                idy = idx + 1
                while idy < sentence_len:
                    char = sentence[idy]
                    if char not in self.non_word_boundaries:
                        break
                    idy += 1
                idx = idy
            # if we are end of sentence and have a sequence discovered
            if idx + 1 >= sentence_len:
                if self._keyword in current_dict:
                    sequence_found = current_dict[self._keyword]
                    keywords_extracted.append((sequence_found, sequence_start_pos, sentence_len))
            idx += 1
            if reset_current_dict:
                reset_current_dict = False
                sequence_start_pos = idx
        if span_info:
            return keywords_extracted
        return [value[0] for value in keywords_extracted]

    def replace_keywords(self, sentence):
        """Searches in the string for all keywords present in corpus.
        Keywords present are replaced by the clean name and a new string is returned.

        Args:
            sentence (str): Line of text where we will replace keywords

        Returns:
            new_sentence (str): Line of text with replaced keywords

        Examples:
            >>> from flashtext import KeywordProcessor
            >>> keyword_processor = KeywordProcessor()
            >>> keyword_processor.add_keyword('Big Apple', 'New York')
            >>> keyword_processor.add_keyword('Bay Area')
            >>> new_sentence = keyword_processor.replace_keywords('I love Big Apple and bay area.')
            >>> new_sentence
            >>> 'I love New York and Bay Area.'

        """
        if not sentence:
            # if sentence is empty or none just return the same.
            return sentence
        new_sentence = ''
        orig_sentence = sentence
        if not self.case_sensitive:
            sentence = sentence.lower()
        current_word = ''
        current_dict = self.keyword_trie_dict
        current_white_space = ''
        sequence_end_pos = 0
        idx = 0
        sentence_len = len(sentence)
        while idx < sentence_len:
            char = sentence[idx]
            current_word += orig_sentence[idx]
            # when we reach whitespace
            if char not in self.non_word_boundaries:
                current_white_space = char
                # if end is present in current_dict
                if self._keyword in current_dict or char in current_dict:
                    # update longest sequence found
                    sequence_found = None
                    longest_sequence_found = None
                    is_longer_seq_found = False
                    if self._keyword in current_dict:
                        sequence_found = current_dict[self._keyword]
                        longest_sequence_found = current_dict[self._keyword]
                        sequence_end_pos = idx

                    # re look for longest_sequence from this position
                    if char in current_dict:
                        current_dict_continued = current_dict[char]
                        current_word_continued = current_word
                        idy = idx + 1
                        while idy < sentence_len:
                            inner_char = sentence[idy]
                            current_word_continued += orig_sentence[idy]
                            if inner_char not in self.non_word_boundaries and self._keyword in current_dict_continued:
                                # update longest sequence found
                                current_white_space = inner_char
                                longest_sequence_found = current_dict_continued[self._keyword]
                                sequence_end_pos = idy
                                is_longer_seq_found = True
                            if inner_char in current_dict_continued:
                                current_dict_continued = current_dict_continued[inner_char]
                            else:
                                break
                            idy += 1
                        else:
                            # end of sentence reached.
                            if self._keyword in current_dict_continued:
                                # update longest sequence found
                                current_white_space = ''
                                longest_sequence_found = current_dict_continued[self._keyword]
                                sequence_end_pos = idy
                                is_longer_seq_found = True
                        if is_longer_seq_found:
                            idx = sequence_end_pos
                            current_word = current_word_continued
                    current_dict = self.keyword_trie_dict
                    if longest_sequence_found:
                        new_sentence += longest_sequence_found + current_white_space
                        current_word = ''
                        current_white_space = ''
                    else:
                        new_sentence += current_word
                        current_word = ''
                        current_white_space = ''
                else:
                    # we reset current_dict
                    current_dict = self.keyword_trie_dict
                    new_sentence += current_word
                    current_word = ''
                    current_white_space = ''
            elif char in current_dict:
                # we can continue from this char
                current_dict = current_dict[char]
            else:
                # we reset current_dict
                current_dict = self.keyword_trie_dict
                # skip to end of word
                idy = idx + 1
                while idy < sentence_len:
                    char = sentence[idy]
                    current_word += orig_sentence[idy]
                    if char not in self.non_word_boundaries:
                        break
                    idy += 1
                idx = idy
                new_sentence += current_word
                current_word = ''
                current_white_space = ''
            # if we are end of sentence and have a sequence discovered
            if idx + 1 >= sentence_len:
                if self._keyword in current_dict:
                    sequence_found = current_dict[self._keyword]
                    new_sentence += sequence_found
                else:
                    new_sentence += current_word
            idx += 1
        return new_sentence



# -*- coding: utf-8 -*-
# Author : Jesper
# Date：   2020/6/30 21:19
# Description :
import json
import pickle
from nlp_utils_ch.utils.logger import log
from nlp_utils_ch.data_process.csv_process import read_csv_data, save_csv_data


def save_pickle(data, file_path):
	'''
	保存成pickle文件
	:param data:
	:param file_name:
	:param pickle_path:
	:return:
	'''
	with open(file_path, 'wb') as f:
		pickle.dump(data, f)


def load_pickle(input_file):
	'''
	读取pickle文件
	:param pickle_path:
	:param file_name:
	:return:
	'''
	with open(str(input_file), 'rb') as f:
		data = pickle.load(f)
	return data


def save_json(data, file_path, ensure_ascii=False, indent=4):
	'''
	保存成json文件
	:param data:
	:param json_path:
	:param file_name:
	:return:
	'''

	with open(str(file_path), 'w') as f:
		json.dump(data, f, ensure_ascii=False, indent=4)


def load_json(file_path):
	'''
	加载json文件
	:param json_path:
	:param file_name:
	:return:
	'''
	with open(str(file_path), 'r') as f:
		data = json.load(f)
	return data


## 解决excel直接复制到csv，因换行,tab符号导致顺序错乱
def excel_to_csv(src_path, tgt_path, replace_chars=["\n", "\t"],
				 header=True,
				 content_col="content",
				 src_file_type="excel"):
	""":arg
	src_file_type: excel/csv
	"""
	# import 写在函数内，避免其它调用时
	import pandas as pd
	if src_file_type == "excel":
		raw_data = pd.read_excel(src_path, encoding="utf-8", errors='ignore')
	elif src_file_type == "csv":
		raw_data = pd.read_csv(src_path, encoding="utf-8")
	else:
		raise NotImplemented
	log.info("raw_data count:{}".format(len(raw_data)))
	col_names = raw_data.columns.values
	res_data = list()
	for i in range(len(raw_data)):
		line_list = list()
		for _col_name in col_names:
			_value = str(raw_data[_col_name][i])
			if _col_name == content_col:
				for _char in replace_chars:
					_value = _value.replace(_char, " ")
			line_list.append(_value)
		res_data.append("\t".join(line_list))

	with open(tgt_path, "w") as f:
		for _line in res_data:
			# for _char in replace_chars:
			_line = _line.replace("\n", " ")
			f.write(_line + "\n")


def csv_to_tsv(src_path, tgt_path):
	"""
	csv转为tsv，并去除tab符和换符
	"""
	import pandas as pd
	data = pd.read_csv(src_path)
	col_names = data.columns
	for _col_name in col_names:
		data[_col_name] = data[_col_name].apply(lambda x: str(x).strip().replace("'\n", " ").
												replace("'\t", " ").replace("'\r", " ").
												replace("'\n\r", " "))
	data.to_csv(tgt_path, sep="\t", index=False)


def sort_big_file_data(src_path, tgt_path, field):
	data = read_csv_data(src_path)
	data = sort_data(data, field)
	save_csv_data(data, tgt_path)


def sort_data(data, field):
	data = sorted(data, key=lambda x: x[field], reverse=True)
	return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
This module implements a simple conversion and localization between simplified and traditional Chinese using tables from MediaWiki.
It doesn't contains a segmentation function and uses maximal forward matching, so it's simple.
For a complete and accurate solution, see OpenCC.
For Chinese segmentation, see Jieba.

    >>> print(convert('我幹什麼不干你事。', 'zh-cn'))
    我干什么不干你事。
    >>> print(convert('人体内存在很多微生物', 'zh-tw'))
    人體內存在很多微生物

Support MediaWiki's convertion format:

    >>> print(convert_for_mw('在现代，机械计算-{}-机的应用已经完全被电子计算-{}-机所取代', 'zh-hk'))
    在現代，機械計算機的應用已經完全被電子計算機所取代
    >>> print(convert_for_mw('-{zh-hant:資訊工程;zh-hans:计算机工程学;}-是电子工程的一个分支，主要研究计算机软硬件和二者间的彼此联系。', 'zh-tw'))
    資訊工程是電子工程的一個分支，主要研究計算機軟硬體和二者間的彼此聯繫。
    >>> print(convert_for_mw('張國榮曾在英國-{zh:利兹;zh-hans:利兹;zh-hk:列斯;zh-tw:里茲}-大学學習。', 'zh-sg'))
    张国荣曾在英国利兹大学学习。

"""
# Only Python3 can pass the doctest here due to unicode problems.
__version__ = '1.4.0'

import os
import sys
import re
import json

try:
    from pkg_resources import resource_stream
    get_module_res = lambda *res: resource_stream(__name__, os.path.join(*res))
except ImportError:
    get_module_res = lambda *res: open(os.path.normpath(
        os.path.join(os.getcwd(), os.path.dirname(__file__), *res)), 'rb')

# Locale fallback order lookup dictionary
Locales = {
    'zh-cn': ('zh-cn', 'zh-hans', 'zh-sg', 'zh'),
    'zh-hk': ('zh-hk', 'zh-hant', 'zh-tw', 'zh'),
    'zh-tw': ('zh-tw', 'zh-hant', 'zh-hk', 'zh'),
    'zh-sg': ('zh-sg', 'zh-hans', 'zh-cn', 'zh'),
    'zh-my': ('zh-my', 'zh-sg', 'zh-hans', 'zh-cn', 'zh'),
    'zh-mo': ('zh-mo', 'zh-hk', 'zh-hant', 'zh-tw', 'zh'),
    'zh-hant': ('zh-hant', 'zh-tw', 'zh-hk', 'zh'),
    'zh-hans': ('zh-hans', 'zh-cn', 'zh-sg', 'zh'),
    'zh': ('zh',) # special value for no conversion
}

_DEFAULT_DICT = "zhcdict.json"
DICTIONARY = _DEFAULT_DICT

zhcdicts = None
dict_zhcn = None
dict_zhsg = None
dict_zhtw = None
dict_zhhk = None
pfsdict = {}

RE_langconv = re.compile(r'(-\{|\}-)')
RE_splitflag = re.compile(r'\s*\|\s*')
RE_splitmap = re.compile(r'\s*;\s*')
RE_splituni = re.compile(r'\s*=>\s*')
RE_splitpair = re.compile(r'\s*:\s*')

def loaddict(filename=DICTIONARY):
    """
    Load the dictionary from a specific JSON file.
    """
    global zhcdicts
    if zhcdicts:
        return
    if filename == _DEFAULT_DICT:
        zhcdicts = json.loads(get_module_res(filename).read().decode('utf-8'))
    else:
        with open(filename, 'rb') as f:
            zhcdicts = json.loads(f.read().decode('utf-8'))
    zhcdicts['SIMPONLY'] = frozenset(zhcdicts['SIMPONLY'])
    zhcdicts['TRADONLY'] = frozenset(zhcdicts['TRADONLY'])

def getdict(locale):
    """
    Generate or get convertion dict cache for certain locale.
    Dictionaries are loaded on demand.
    """
    global zhcdicts, dict_zhcn, dict_zhsg, dict_zhtw, dict_zhhk, pfsdict
    if zhcdicts is None:
        loaddict(DICTIONARY)
    if locale == 'zh-cn':
        if dict_zhcn:
            got = dict_zhcn
        else:
            dict_zhcn = zhcdicts['zh2Hans'].copy()
            dict_zhcn.update(zhcdicts['zh2CN'])
            got = dict_zhcn
    elif locale == 'zh-tw':
        if dict_zhtw:
            got = dict_zhtw
        else:
            dict_zhtw = zhcdicts['zh2Hant'].copy()
            dict_zhtw.update(zhcdicts['zh2TW'])
            got = dict_zhtw
    elif locale == 'zh-hk' or locale == 'zh-mo':
        if dict_zhhk:
            got = dict_zhhk
        else:
            dict_zhhk = zhcdicts['zh2Hant'].copy()
            dict_zhhk.update(zhcdicts['zh2HK'])
            got = dict_zhhk
    elif locale == 'zh-sg' or locale == 'zh-my':
        if dict_zhsg:
            got = dict_zhsg
        else:
            dict_zhsg = zhcdicts['zh2Hans'].copy()
            dict_zhsg.update(zhcdicts['zh2SG'])
            got = dict_zhsg
    elif locale == 'zh-hans':
        got = zhcdicts['zh2Hans']
    elif locale == 'zh-hant':
        got = zhcdicts['zh2Hant']
    else:
        got = {}
    if locale not in pfsdict:
        pfsdict[locale] = getpfset(got)
    return got

def getpfset(convdict):
    pfset = []
    for word in convdict:
        for ch in range(len(word)):
            pfset.append(word[:ch+1])
    return frozenset(pfset)

def issimp(s, full=False):
    """
    Detect text is whether Simplified Chinese or Traditional Chinese.
    Returns True for Simplified; False for Traditional; None for unknown.
    If full=False, it returns once first simplified- or traditional-only
    character is encountered, so it's for quick and rough identification;
    else, it compares the count and returns the most likely one.
    Use `is` (True/False/None) to check the result.

    `s` must be unicode (Python 2) or str (Python 3), or you'll get None.
    """
    if zhcdicts is None:
        loaddict(DICTIONARY)
    simp, trad = 0, 0
    if full:
        for ch in s:
            if ch in zhcdicts['SIMPONLY']:
                simp += 1
            elif ch in zhcdicts['TRADONLY']:
                trad += 1
        if simp > trad:
            return True
        elif simp < trad:
            return False
        else:
            return None
    else:
        for ch in s:
            if ch in zhcdicts['SIMPONLY']:
                return True
            elif ch in zhcdicts['TRADONLY']:
                return False
        return None

def fallback(locale, mapping):
    for l in Locales[locale]:
        if l in mapping:
            return mapping[l]
    return convert(tuple(mapping.values())[0], locale)

def convtable2dict(convtable, locale, update=None):
    """
    Convert a list of conversion dict to a dict for a certain locale.

    >>> sorted(convtable2dict([{'zh-hk': '列斯', 'zh-hans': '利兹', 'zh': '利兹', 'zh-tw': '里茲'}, {':uni': '巨集', 'zh-cn': '宏'}], 'zh-cn').items())
    [('列斯', '利兹'), ('利兹', '利兹'), ('巨集', '宏'), ('里茲', '利兹')]
    """
    rdict = update.copy() if update else {}
    for r in convtable:
        if ':uni' in r:
            if locale in r:
                rdict[r[':uni']] = r[locale]
        elif locale[:-1] == 'zh-han':
            if locale in r:
                for word in r.values():
                    rdict[word] = r[locale]
        else:
            v = fallback(locale, r)
            for word in r.values():
                rdict[word] = v
    return rdict

def tokenize(s, locale, update=None):
    """
    Tokenize `s` according to corresponding locale dictionary.
    Don't use this for serious text processing.
    """
    zhdict = getdict(locale)
    pfset = pfsdict[locale]
    if update:
        zhdict = zhdict.copy()
        zhdict.update(update)
        newset = set()
        for word in update:
            for ch in range(len(word)):
                newset.add(word[:ch+1])
        pfset = pfset | newset
    ch = []
    N = len(s)
    pos = 0
    while pos < N:
        i = pos
        frag = s[pos]
        maxword = None
        maxpos = 0
        while i < N and frag in pfset:
            if frag in zhdict:
                maxword = frag
                maxpos = i
            i += 1
            frag = s[pos:i+1]
        if maxword is None:
            maxword = s[pos]
            pos += 1
        else:
            pos = maxpos + 1
        ch.append(maxword)
    return ch

def convert(s, locale, update=None):
    """
    Main convert function.

    :param s: must be `unicode` (Python 2) or `str` (Python 3).
    :param locale: should be one of ``('zh-hans', 'zh-hant', 'zh-cn', 'zh-sg'
                               'zh-tw', 'zh-hk', 'zh-my', 'zh-mo')``.
    :param update: a dict which updates the conversion table, eg.
        ``{'from1': 'to1', 'from2': 'to2'}``

    >>> print(convert('我幹什麼不干你事。', 'zh-cn'))
    我干什么不干你事。
    >>> print(convert('我幹什麼不干你事。', 'zh-cn', {'不干': '不幹'}))
    我干什么不幹你事。
    >>> print(convert('人体内存在很多微生物', 'zh-tw'))
    人體內存在很多微生物
    """
    if locale == 'zh' or locale not in Locales:
        # "no conversion"
        return s
    zhdict = getdict(locale)
    pfset = pfsdict[locale]
    newset = set()
    if update:
        # TODO: some sort of caching
        #zhdict = zhdict.copy()
        #zhdict.update(update)
        newset = set()
        for word in update:
            for ch in range(len(word)):
                newset.add(word[:ch+1])
        #pfset = pfset | newset
    ch = []
    N = len(s)
    pos = 0
    while pos < N:
        i = pos
        frag = s[pos]
        maxword = None
        maxpos = 0
        while i < N and (frag in pfset or frag in newset):
            if update and frag in update:
                maxword = update[frag]
                maxpos = i
            elif frag in zhdict:
                maxword = zhdict[frag]
                maxpos = i
            i += 1
            frag = s[pos:i+1]
        if maxword is None:
            maxword = s[pos]
            pos += 1
        else:
            pos = maxpos + 1
        ch.append(maxword)
    return ''.join(ch)

def convert_for_mw(s, locale, update=None):
    """
    Recognizes MediaWiki's human conversion format.
    Use locale='zh' for no conversion.

    Reference: (all tests passed)
    https://zh.wikipedia.org/wiki/Help:高级字词转换语法
    https://www.mediawiki.org/wiki/Writing_systems/Syntax

    >>> print(convert_for_mw('在现代，机械计算-{}-机的应用已经完全被电子计算-{}-机所取代', 'zh-hk'))
    在現代，機械計算機的應用已經完全被電子計算機所取代
    >>> print(convert_for_mw('-{zh-hant:資訊工程;zh-hans:计算机工程学;}-是电子工程的一个分支，主要研究计算机软硬件和二者间的彼此联系。', 'zh-tw'))
    資訊工程是電子工程的一個分支，主要研究計算機軟硬體和二者間的彼此聯繫。
    >>> print(convert_for_mw('張國榮曾在英國-{zh:利兹;zh-hans:利兹;zh-hk:列斯;zh-tw:里茲}-大学學習。', 'zh-hant'))
    張國榮曾在英國里茲大學學習。
    >>> print(convert_for_mw('張國榮曾在英國-{zh:利兹;zh-hans:利兹;zh-hk:列斯;zh-tw:里茲}-大学學習。', 'zh-sg'))
    张国荣曾在英国利兹大学学习。
    >>> convert_for_mw('-{zh-hant:;\\nzh-cn:}-', 'zh-tw') == ''
    True
    >>> print(convert_for_mw('毫米(毫公分)，符號mm，是長度單位和降雨量單位，-{zh-hans:台湾作-{公釐}-或-{公厘}-;zh-hant:港澳和大陸稱為-{毫米}-（台灣亦有使用，但較常使用名稱為毫公分）;zh-mo:台灣作-{公釐}-或-{公厘}-;zh-hk:台灣作-{公釐}-或-{公厘}-;}-。', 'zh-tw'))
    毫米(毫公分)，符號mm，是長度單位和降雨量單位，港澳和大陸稱為毫米（台灣亦有使用，但較常使用名稱為毫公分）。
    >>> print(convert_for_mw('毫米(毫公分)，符號mm，是長度單位和降雨量單位，-{zh-hans:台湾作-{公釐}-或-{公厘}-;zh-hant:港澳和大陸稱為-{毫米}-（台灣亦有使用，但較常使用名稱為毫公分）;zh-mo:台灣作-{公釐}-或-{公厘}-;zh-hk:台灣作-{公釐}-或-{公厘}-;}-。', 'zh-cn'))
    毫米(毫公分)，符号mm，是长度单位和降雨量单位，台湾作公釐或公厘。
    >>> print(convert_for_mw('毫米(毫公分)，符號mm，是長度單位和降雨量單位，-{zh-hans:台湾作-{公釐}-或-{公厘}-;zh-hant:港澳和大陸稱為-{毫米}-（台灣亦有使用，但較常使用名稱為毫公分）;zh-mo:台灣作-{公釐}-或-{公厘}-;zh-hk:台灣作-{公釐}-或-{公厘', 'zh-hk'))  # unbalanced test
    毫米(毫公分)，符號mm，是長度單位和降雨量單位，台灣作公釐或公厘
    >>> print(convert_for_mw('报头的“-{參攷消息}-”四字摘自鲁迅笔迹-{zh-hans:，“-{參}-”是“-{参}-”的繁体字，读音cān，与简体的“-{参}-”字相同；;zh-hant:，;}-“-{攷}-”是“考”的异体字，读音kǎo，与“考”字相同。', 'zh-tw'))
    報頭的「參攷消息」四字摘自魯迅筆跡，「攷」是「考」的異體字，讀音kǎo，與「考」字相同。
    >>> print(convert_for_mw('报头的“-{參攷消息}-”四字摘自鲁迅笔迹-{zh-hans:，“-{參}-”是“-{参}-”的繁体字，读音cān，与简体的“-{参}-”字相同；;zh-hant:，;}-“-{攷}-”是“考”的异体字，读音kǎo，与“考”字相同。', 'zh-cn'))
    报头的“參攷消息”四字摘自鲁迅笔迹，“參”是“参”的繁体字，读音cān，与简体的“参”字相同；“攷”是“考”的异体字，读音kǎo，与“考”字相同。
    >>> print(convert_for_mw('{{Col-break}}--&gt;', 'zh-hant'))
    {{Col-break}}--&gt;
    """
    ch = []
    rules = []
    ruledict = update.copy() if update else {}
    nested = 0
    block = ''
    for frag in RE_langconv.split(s):
        if frag == '-{':
            nested += 1
            block += frag
        elif frag == '}-':
            if not nested:
                # bogus }-
                ch.append(frag)
                continue
            block += frag
            nested -= 1
            if nested:
                continue
            newrules = []
            delim = RE_splitflag.split(block[2:-2].strip(' \t\n\r\f\v;'))
            if len(delim) == 1:
                flag = None
                mapping = RE_splitmap.split(delim[0])
            else:
                flag = RE_splitmap.split(delim[0].strip(' \t\n\r\f\v;'))
                mapping = RE_splitmap.split(delim[1])
            rule = {}
            for m in mapping:
                uni = RE_splituni.split(m)
                if len(uni) == 1:
                    pair = RE_splitpair.split(uni[0])
                else:
                    if rule:
                        newrules.append(rule)
                        rule = {':uni': uni[0]}
                    else:
                        rule[':uni'] = uni[0]
                    pair = RE_splitpair.split(uni[1])
                if len(pair) == 1:
                    rule['zh'] = convert_for_mw(pair[0], 'zh', ruledict)
                else:
                    rule[pair[0]] = convert_for_mw(pair[1], pair[0], ruledict)
            newrules.append(rule)
            if not flag:
                ch.append(fallback(locale, newrules[0]))
            elif any(ch in flag for ch in 'ATRD-HN'):
                for f in flag:
                    # A: add rule for convert code (all text convert)
                    # H: Insert a conversion rule without output
                    if f in ('A', 'H'):
                        for r in newrules:
                            if not r in rules:
                                rules.append(r)
                        if f == 'A':
                            if ':uni' in r:
                                if locale in r:
                                    ch.append(r[locale])
                                else:
                                    ch.append(convert(r[':uni'], locale))
                            else:
                                ch.append(fallback(locale, newrules[0]))
                    # -: remove convert
                    elif f == '-':
                        for r in newrules:
                            try:
                                rules.remove(r)
                            except ValueError:
                                pass
                    # D: convert description (useless)
                    #elif f == 'D':
                        #ch.append('; '.join(': '.join(x) for x in newrules[0].items()))
                    # T: title convert (useless)
                    # R: raw content (implied above)
                    # N: current variant name (useless)
                    #elif f == 'N':
                        #ch.append(locale)
                ruledict = convtable2dict(rules, locale, update)
            else:
                fblimit = frozenset(flag) & frozenset(Locales[locale])
                limitedruledict = update.copy() if update else {}
                for r in rules:
                    if ':uni' in r:
                        if locale in r:
                            limitedruledict[r[':uni']] = r[locale]
                    else:
                        v = None
                        for l in Locales[locale]:
                            if l in r and l in fblimit:
                                v = r[l]
                                break
                        for word in r.values():
                            limitedruledict[word] = v if v else convert(word, locale)
                ch.append(convert(delim[1], locale, limitedruledict))
            block = ''
        elif nested:
            block += frag
        else:
            ch.append(convert(frag, locale, ruledict))
    if nested:
        # unbalanced
        ch.append(convert_for_mw(block + '}-'*nested, locale, ruledict))
    return ''.join(ch)

def test_convert_mw(locale, update=None):
    s = ('英國-{zh:利兹;zh-hans:利兹;zh-hk:列斯;zh-tw:里茲}-大学\n'
        '-{zh-hans:计算机; zh-hant:電腦;}-\n'
        '-{H|巨集=>zh-cn:宏;}-\n'
        '测试：巨集、宏\n'
        '-{简体字繁體字}-\n'
        '北-{}-韓、北朝-{}-鲜\n'
        '-{H|zh-cn:博客; zh-hk:網誌; zh-tw:部落格;}-\n'
        '测试：博客、網誌、部落格\n'
        '-{A|zh-cn:博客; zh-hk:網誌; zh-tw:部落格;}-\n'
        '测试：博客、網誌、部落格\n'
        '-{H|zh-cn:博客; zh-hk:網誌; zh-tw:部落格;}-\n'
        '测试1：博客、網誌、部落格\n'
        '-{-|zh-cn:博客; zh-hk:網誌; zh-tw:部落格;}-\n'
        '测试2：博客、網誌、部落格\n'
        '-{T|zh-cn:汤姆·汉克斯; zh-hk:湯·漢斯; zh-tw:湯姆·漢克斯;}-\n'
        '-{D|zh-cn:汤姆·汉克斯; zh-hk:湯·漢斯; zh-tw:湯姆·漢克斯;}-\n'
        '-{H|zh-cn:博客; zh-hk:網誌; zh-tw:部落格;}-\n'
        '测试1：-{zh;zh-hans;zh-hant|博客、網誌、部落格}-\n'
        '测试2：-{zh;zh-cn;zh-hk|博客、網誌、部落格}-')
    return convert_for_mw(s, locale, update)

def main():
    """
    Simple stdin/stdout interface.
    """
    if len(sys.argv) == 2 and sys.argv[1] in Locales:
        locale = sys.argv[1]
        convertfunc = convert
    elif len(sys.argv) == 3 and sys.argv[1] == '-w' and sys.argv[2] in Locales:
        locale = sys.argv[2]
        convertfunc = convert_for_mw
    else:
        thisfile = __file__ if __name__ == '__main__' else 'python -mzhconv'
        print("usage: %s [-w] {zh-cn|zh-tw|zh-hk|zh-sg|zh-hans|zh-hant|zh} < input > output" % thisfile)
        sys.exit(1)

    loaddict()
    ln = sys.stdin.readline()
    while ln:
        l = ln.rstrip('\r\n')
        if sys.version_info[0] < 3:
            l = unicode(l, 'utf-8')
        res = convertfunc(l, locale)
        if sys.version_info[0] < 3:
            print(res.encode('utf-8'))
        else:
            print(res)
        ln = sys.stdin.readline()

if __name__ == '__main__':
    main()



#!user/bin/python
# _*_ coding: utf-8 _*_
# @Author      :   Jesper
# @Time        :   2020/8/18 11:33
# @Description :

from __future__ import absolute_import
import six
import itertools
from copy import deepcopy
# import rouge.rouge_score as rouge_score
import io
import os


class FilesRouge:
    def __init__(self, *args, **kwargs):
        """See the `Rouge` class for args
        """
        self.rouge = Rouge(*args, **kwargs)

    def _check_files(self, hyp_path, ref_path):
        assert(os.path.isfile(hyp_path))
        assert(os.path.isfile(ref_path))

        def line_count(path):
            count = 0
            with open(path, "rb") as f:
                for line in f:
                    count += 1
            return count

        hyp_lc = line_count(hyp_path)
        ref_lc = line_count(ref_path)
        assert(hyp_lc == ref_lc)

    def get_scores(self, hyp_path, ref_path, avg=False, ignore_empty=False):
        """Calculate ROUGE scores between each pair of
        lines (hyp_file[i], ref_file[i]).
        Args:
          * hyp_path: hypothesis file path
          * ref_path: references file path
          * avg (False): whether to get an average scores or a list
        """
        self._check_files(hyp_path, ref_path)

        with io.open(hyp_path, encoding="utf-8", mode="r") as hyp_file:
            hyps = [line[:-1] for line in hyp_file]

        with io.open(ref_path, encoding="utf-8", mode="r") as ref_file:
            refs = [line[:-1] for line in ref_file]

        return self.rouge.get_scores(hyps, refs, avg=avg,
                                     ignore_empty=ignore_empty)


class Rouge:
    DEFAULT_METRICS = ["rouge-1", "rouge-2", "rouge-l"]
    AVAILABLE_METRICS = {
        "rouge-1": lambda hyp, ref, **k: rouge_n(hyp, ref, 1, **k),
        "rouge-2": lambda hyp, ref, **k: rouge_n(hyp, ref, 2, **k),
        "rouge-l": lambda hyp, ref, **k:
            rouge_l_summary_level(hyp, ref, **k),
    }
    DEFAULT_STATS = ["f", "p", "r"]
    AVAILABLE_STATS = ["f", "p", "r"]

    def __init__(self, metrics=None, stats=None, return_lengths=False,
                 raw_results=False, exclusive=False):
        self.return_lengths = return_lengths
        self.raw_results = raw_results
        self.exclusive = exclusive

        if metrics is not None:
            self.metrics = [m.lower() for m in metrics]

            for m in self.metrics:
                if m not in Rouge.AVAILABLE_METRICS:
                    raise ValueError("Unknown metric '%s'" % m)
        else:
            self.metrics = Rouge.DEFAULT_METRICS

        if self.raw_results:
            self.stats = ["hyp", "ref", "overlap"]
        else:
            if stats is not None:
                self.stats = [s.lower() for s in stats]

                for s in self.stats:
                    if s not in Rouge.AVAILABLE_STATS:
                        raise ValueError("Unknown stat '%s'" % s)
            else:
                self.stats = Rouge.DEFAULT_STATS

    def get_scores(self, hyps, refs, avg=False, ignore_empty=False):
        if isinstance(hyps, six.string_types):
            hyps, refs = [hyps], [refs]

        if ignore_empty:
            # Filter out hyps of 0 length
            hyps_and_refs = zip(hyps, refs)
            hyps_and_refs = [_ for _ in hyps_and_refs
                             if len(_[0]) > 0
                             and len(_[1]) > 0]
            hyps, refs = zip(*hyps_and_refs)

        assert(isinstance(hyps, type(refs)))
        assert(len(hyps) == len(refs))

        if not avg:
            return self._get_scores(hyps, refs)
        return self._get_avg_scores(hyps, refs)

    def _get_scores(self, hyps, refs):
        scores = []
        for hyp, ref in zip(hyps, refs):
            sen_score = {}

            hyp = [" ".join(_.split()) for _ in hyp.split(".") if len(_) > 0]
            ref = [" ".join(_.split()) for _ in ref.split(".") if len(_) > 0]

            for m in self.metrics:
                fn = Rouge.AVAILABLE_METRICS[m]
                sc = fn(
                    hyp,
                    ref,
                    raw_results=self.raw_results,
                    exclusive=self.exclusive)
                sen_score[m] = {s: sc[s] for s in self.stats}

            if self.return_lengths:
                lengths = {
                    "hyp": len(" ".join(hyp).split()),
                    "ref": len(" ".join(ref).split())
                }
                sen_score["lengths"] = lengths
            scores.append(sen_score)
        return scores

    def _get_avg_scores(self, hyps, refs):
        scores = {m: {s: 0 for s in self.stats} for m in self.metrics}
        if self.return_lengths:
            scores["lengths"] = {"hyp": 0, "ref": 0}

        count = 0
        for (hyp, ref) in zip(hyps, refs):
            hyp = [" ".join(_.split()) for _ in hyp.split(".") if len(_) > 0]
            ref = [" ".join(_.split()) for _ in ref.split(".") if len(_) > 0]

            for m in self.metrics:
                fn = Rouge.AVAILABLE_METRICS[m]
                sc = fn(hyp, ref, exclusive=self.exclusive)
                scores[m] = {s: scores[m][s] + sc[s] for s in self.stats}

            if self.return_lengths:
                scores["lengths"]["hyp"] += len(" ".join(hyp).split())
                scores["lengths"]["ref"] += len(" ".join(ref).split())

            count += 1
        avg_scores = {
            m: {s: scores[m][s] / count for s in self.stats}
            for m in self.metrics
        }

        if self.return_lengths:
            avg_scores["lengths"] = {
                k: scores["lengths"][k] / count
                for k in ["hyp", "ref"]
            }

        return avg_scores



class Ngrams(object):
    """
        Ngrams datastructure based on `set` or `list`
        depending in `exclusive`
    """

    def __init__(self, ngrams={}, exclusive=True):
        if exclusive:
            self._ngrams = set(ngrams)
        else:
            self._ngrams = list(ngrams)
        self.exclusive = exclusive

    def add(self, o):
        if self.exclusive:
            self._ngrams.add(o)
        else:
            self._ngrams.append(o)

    def __len__(self):
        return len(self._ngrams)

    def intersection(self, o):
        if self.exclusive:
            inter_set = self._ngrams.intersection(o._ngrams)
            return Ngrams(inter_set, exclusive=True)
        else:
            other_list = deepcopy(o._ngrams)
            inter_list = []

            for e in self._ngrams:
                try:
                    i = other_list.index(e)
                except ValueError:
                    continue
                other_list.pop(i)
                inter_list.append(e)
            return Ngrams(inter_list, exclusive=False)

    def union(self, *ngrams):
        if self.exclusive:
            union_set = self._ngrams
            for o in ngrams:
                union_set = union_set.union(o._ngrams)
            return Ngrams(union_set, exclusive=True)
        else:
            union_list = deepcopy(self._ngrams)
            for o in ngrams:
                union_list.extend(o._ngrams)
            return Ngrams(union_list, exclusive=False)


def _get_ngrams(n, text, exclusive=True):
    """Calcualtes n-grams.
    Args:
      n: which n-grams to calculate
      text: An array of tokens
    Returns:
      A set of n-grams
    """
    ngram_set = Ngrams(exclusive=exclusive)
    text_length = len(text)
    max_index_ngram_start = text_length - n
    for i in range(max_index_ngram_start + 1):
        ngram_set.add(tuple(text[i:i + n]))
    return ngram_set


def _split_into_words(sentences):
    """Splits multiple sentences into words and flattens the result"""
    return list(itertools.chain(*[_.split(" ") for _ in sentences]))


def _get_word_ngrams(n, sentences, exclusive=True):
    """Calculates word n-grams for multiple sentences.
    """
    assert len(sentences) > 0
    assert n > 0

    words = _split_into_words(sentences)
    return _get_ngrams(n, words, exclusive=exclusive)


def _len_lcs(x, y):
    """
    Returns the length of the Longest Common Subsequence between sequences x
    and y.
    Source: http://www.algorithmist.com/index.php/Longest_Common_Subsequence
    Args:
      x: sequence of words
      y: sequence of words
    Returns
      integer: Length of LCS between x and y
    """
    table = _lcs(x, y)
    n, m = len(x), len(y)
    return table[n, m]


def _lcs(x, y):
    """
    Computes the length of the longest common subsequence (lcs) between two
    strings. The implementation below uses a DP programming algorithm and runs
    in O(nm) time where n = len(x) and m = len(y).
    Source: http://www.algorithmist.com/index.php/Longest_Common_Subsequence
    Args:
      x: collection of words
      y: collection of words
    Returns:
      Table of dictionary of coord and len lcs
    """
    n, m = len(x), len(y)
    table = dict()
    for i in range(n + 1):
        for j in range(m + 1):
            if i == 0 or j == 0:
                table[i, j] = 0
            elif x[i - 1] == y[j - 1]:
                table[i, j] = table[i - 1, j - 1] + 1
            else:
                table[i, j] = max(table[i - 1, j], table[i, j - 1])
    return table


def _recon_lcs(x, y, exclusive=True):
    """
    Returns the Longest Subsequence between x and y.
    Source: http://www.algorithmist.com/index.php/Longest_Common_Subsequence
    Args:
      x: sequence of words
      y: sequence of words
    Returns:
      sequence: LCS of x and y
    """
    i, j = len(x), len(y)
    table = _lcs(x, y)

    def _recon(i, j):
        """private recon calculation"""
        if i == 0 or j == 0:
            return []
        elif x[i - 1] == y[j - 1]:
            return _recon(i - 1, j - 1) + [(x[i - 1], i)]
        elif table[i - 1, j] > table[i, j - 1]:
            return _recon(i - 1, j)
        else:
            return _recon(i, j - 1)

    recon_list = list(map(lambda x: x[0], _recon(i, j)))
    return Ngrams(recon_list, exclusive=exclusive)
    return recon_tuple


def multi_rouge_n(sequences, scores_ids, n=2, exclusive=True):
    """
    Efficient way to compute highly repetitive scoring
    i.e. sequences are involved multiple time
    Args:
        sequences(list[str]): list of sequences (either hyp or ref)
        scores_ids(list[tuple(int)]): list of pairs (hyp_id, ref_id)
            ie. scores[i] = rouge_n(scores_ids[i][0],
                                    scores_ids[i][1])
    Returns:
        scores: list of length `len(scores_ids)` containing rouge `n`
                scores as a dict with 'f', 'r', 'p'
    Raises:
        KeyError: if there's a value of i in scores_ids that is not in
                  [0, len(sequences)[
    """
    ngrams = [_get_word_ngrams(n, sequence, exclusive=exclusive)
              for sequence in sequences]
    counts = [len(ngram) for ngram in ngrams]

    scores = []
    for hyp_id, ref_id in scores_ids:
        evaluated_ngrams = ngrams[hyp_id]
        evaluated_count = counts[hyp_id]

        reference_ngrams = ngrams[ref_id]
        reference_count = counts[ref_id]

        overlapping_ngrams = evaluated_ngrams.intersection(reference_ngrams)
        overlapping_count = len(overlapping_ngrams)

        scores += [f_r_p_rouge_n(evaluated_count,
                                 reference_count, overlapping_count)]
    return scores


def rouge_n(evaluated_sentences, reference_sentences,
            n=2, raw_results=False, exclusive=True):
    """
    Computes ROUGE-N of two text collections of sentences.
    Sourece: http://research.microsoft.com/en-us/um/people/cyl/download/
    papers/rouge-working-note-v1.3.1.pdf
    Args:
      evaluated_sentences: The sentences that have been picked by the
                           summarizer
      reference_sentences: The sentences from the referene set
      n: Size of ngram.  Defaults to 2.
    Returns:
      A tuple (f1, precision, recall) for ROUGE-N
    Raises:
      ValueError: raises exception if a param has len <= 0
    """
    if len(evaluated_sentences) <= 0:
        raise ValueError("Hypothesis is empty.")
    if len(reference_sentences) <= 0:
        raise ValueError("Reference is empty.")

    evaluated_ngrams = _get_word_ngrams(
        n, evaluated_sentences, exclusive=exclusive)
    reference_ngrams = _get_word_ngrams(
        n, reference_sentences, exclusive=exclusive)
    reference_count = len(reference_ngrams)
    evaluated_count = len(evaluated_ngrams)

    # Gets the overlapping ngrams between evaluated and reference
    overlapping_ngrams = evaluated_ngrams.intersection(reference_ngrams)
    overlapping_count = len(overlapping_ngrams)

    if raw_results:
        o = {
            "hyp": evaluated_count,
            "ref": reference_count,
            "overlap": overlapping_count
        }
        return o
    else:
        return f_r_p_rouge_n(
            evaluated_count, reference_count, overlapping_count)


def f_r_p_rouge_n(evaluated_count, reference_count, overlapping_count):
    # Handle edge case. This isn't mathematically correct, but it's good enough
    if evaluated_count == 0:
        precision = 0.0
    else:
        precision = overlapping_count / evaluated_count

    if reference_count == 0:
        recall = 0.0
    else:
        recall = overlapping_count / reference_count

    f1_score = 2.0 * ((precision * recall) / (precision + recall + 1e-8))

    return {"f": f1_score, "p": precision, "r": recall}


def _union_lcs(evaluated_sentences, reference_sentence,
               prev_union=None, exclusive=True):
    """
    Returns LCS_u(r_i, C) which is the LCS score of the union longest common
    subsequence between reference sentence ri and candidate summary C.
    For example:
    if r_i= w1 w2 w3 w4 w5, and C contains two sentences: c1 = w1 w2 w6 w7 w8
    and c2 = w1 w3 w8 w9 w5, then the longest common subsequence of r_i and c1
    is "w1 w2" and the longest common subsequence of r_i and c2 is "w1 w3 w5".
    The union longest common subsequence of r_i, c1, and c2 is "w1 w2 w3 w5"
    and LCS_u(r_i, C) = 4/5.
    Args:
      evaluated_sentences: The sentences that have been picked by the
                           summarizer
      reference_sentence: One of the sentences in the reference summaries
    Returns:
      float: LCS_u(r_i, C)
    ValueError:
      Raises exception if a param has len <= 0
    """
    if prev_union is None:
        prev_union = Ngrams(exclusive=exclusive)

    if len(evaluated_sentences) <= 0:
        raise ValueError("Collections must contain at least 1 sentence.")

    lcs_union = prev_union
    prev_count = len(prev_union)
    reference_words = _split_into_words([reference_sentence])

    combined_lcs_length = 0
    for eval_s in evaluated_sentences:
        evaluated_words = _split_into_words([eval_s])
        lcs = _recon_lcs(reference_words, evaluated_words, exclusive=exclusive)
        combined_lcs_length += len(lcs)
        lcs_union = lcs_union.union(lcs)

    new_lcs_count = len(lcs_union) - prev_count
    return new_lcs_count, lcs_union


def rouge_l_summary_level(
        evaluated_sentences, reference_sentences, raw_results=False, exclusive=True, **_):
    """
    Computes ROUGE-L (summary level) of two text collections of sentences.
    http://research.microsoft.com/en-us/um/people/cyl/download/papers/
    rouge-working-note-v1.3.1.pdf
    Calculated according to:
    R_lcs = SUM(1, u)[LCS<union>(r_i,C)]/m
    P_lcs = SUM(1, u)[LCS<union>(r_i,C)]/n
    F_lcs = (2*R_lcs*P_lcs) / (R_lcs * P_lcs)
    where:
    SUM(i,u) = SUM from i through u
    u = number of sentences in reference summary
    C = Candidate summary made up of v sentences
    m = number of words in reference summary
    n = number of words in candidate summary
    Args:
      evaluated_sentences: The sentences that have been picked by the
                           summarizer
      reference_sentence: One of the sentences in the reference summaries
    Returns:
      A float: F_lcs
    Raises:
      ValueError: raises exception if a param has len <= 0
    """
    if len(evaluated_sentences) <= 0 or len(reference_sentences) <= 0:
        raise ValueError("Collections must contain at least 1 sentence.")

    # total number of words in reference sentences
    m = len(
        Ngrams(
            _split_into_words(reference_sentences),
            exclusive=exclusive))

    # total number of words in evaluated sentences
    n = len(
        Ngrams(
            _split_into_words(evaluated_sentences),
            exclusive=exclusive))

    # print("m,n %d %d" % (m, n))
    union_lcs_sum_across_all_references = 0
    union = Ngrams(exclusive=exclusive)
    for ref_s in reference_sentences:
        lcs_count, union = _union_lcs(evaluated_sentences,
                                      ref_s,
                                      prev_union=union,
                                      exclusive=exclusive)
        union_lcs_sum_across_all_references += lcs_count

    llcs = union_lcs_sum_across_all_references
    r_lcs = llcs / m
    p_lcs = llcs / n

    f_lcs = 2.0 * ((p_lcs * r_lcs) / (p_lcs + r_lcs + 1e-8))

    if raw_results:
        o = {
            "hyp": n,
            "ref": m,
            "overlap": llcs
        }
        return o
    else:
        return {"f": f_lcs, "p": p_lcs, "r": r_lcs}



# -*- coding: utf-8 -*-
"""
   Author :        Jesper
   Date：          2020/5/25 17:30
   Description :
   Changed by:
"""

import pymysql.cursors
import pymysql
from nlp_utils_ch.utils.logger import log





class mysqlProcess():
	def __init__(self, db_config=None):
		self._connect_mysql(db_config)

	def _connect_mysql(self, db_config):
		log.info("Connect mysql.....")
		if db_config is None:
			log.error("No mysql config.......")

		"""
		db_config = {
			"host": ""
			"port": 3306,
			"user": "",
			"password":""
		}
		"""

		mysql_host = db_config["host"]
		mysql_port = 3306 if "port" not in db_config else db_config["port"],
		mysql_user = db_config["user"]
		mysql_password = db_config["password"]
		mysql_db = db_config["database"]

		log.info("mysql config:{}".format(db_config))
		try:
			log.info("Connect mysql..., database:{}, host:{}".format(mysql_db, mysql_host))
			if mysql_password:
				log.info("password:{}".format(mysql_password))
				self.connection = pymysql.connect(host=mysql_host,
												  user=mysql_user,
												  password=mysql_password,
												  port=mysql_port,
												  db=mysql_db,
												  cursorclass=pymysql.cursors.DictCursor)
			else:
				log.info("password:{}".format(mysql_password))
				self.connection = pymysql.connect(host=mysql_host,
												  user=mysql_user,
												  db=mysql_db,
												  cursorclass=pymysql.cursors.DictCursor)
			log.info("Connect mysql sucess....")
		except Exception as e:
			log.error(e, exc_info=True)

	def insert_data(self, sql_str):
		try:
			with self.connection.cursor() as cursor:
				# Create a new record
				cursor.execute(sql_str)

			# connection is not autocommit by default. So you must commit to save
			# your changes.
			self.connection.commit()
		except Exception as e:
			print('Error e:{}'.format(e))
			self.connection.rollback()

	def insert_many(self, sql_str, val):
		try:
			# 执行sql语句
			with self.connection.cursor() as cursor:
				cursor.executemany(sql_str, val)
			# 提交到数据库执行
			self.connection.commit()
		except Exception as e:
			# 如果发生错误则回滚
			print('Error e:{}'.format(e))
			self.connection.rollback()

	def query_data(self, sql_str):
		try:

			with self.connection.cursor() as cursor:
				# Read a single record
				cursor.execute(sql_str)
				result = cursor.fetchall()

		except Exception as e:
			print('Error e:{}'.format(e))
			log.error("error:{}.....".format(e))
			result = list()
		return result

	def excute_mysql(self, sql_str):
		try:
			with self.connection.cursor() as cursor:
				# Create a new record
				cursor.execute(sql_str)

			# connection is not autocommit by default. So you must commit to save
			# your changes.
			self.connection.commit()
		except Exception as e:
			print('Error e:{}'.format(e))
			self.connection.rollback()

	def close_connection(self):
		"""
		waring: it's necessary
		:return:
		"""
		self.connection.close()


def query_msyql_data(db_config, sql_str):
	mysql_process = mysqlProcess(db_config)
	msyql_data = mysql_process.query_data(sql_str)
	mysql_process.close_connection()
	return msyql_data





#!user/bin/python
# _*_ coding: utf-8 _*_
# @Author      :   Jesper
# @Time        :   2021/7/20  11:13
# @Description :

from nlp_utils_ch.utils.logger import log


def read_csv_data(file_path, col_names=None, header=True, sep="\t", ignore_null=False):
	data = list()
	with open(file_path, "r", encoding='utf-8-sig') as f:
		count = 0
		for line in f.readlines():

			if count == 0 and header is True:
				col_names = line.strip().split(sep)
				count += 1
				continue

			line_list = line.strip("\n").split(sep)
			if ignore_null is True and len(line_list) < len(col_names):
				for i in range(len(col_names) - len(line_list)):
					line_list.append("")

			tmp_dict = dict()
			try:
				for i, col_name in enumerate(col_names):
					tmp_dict[col_name] = line_list[i]
				data.append(tmp_dict)
			except Exception as e:
				log.info("Error line:{}".format(line))
				# log.error(e, exc_info=True)

	log.info("data num:{}".format(len(data)))
	log.info("data example 2:{}".format(data[:2]))
	return data


def save_csv_data(data, file_path, col_names=None, exist_header=True):
	if len(data) == 0:
		log.info("\n\n\ndata is empty!!!!!\n\n\n")
		return

	with open(file_path, "w") as f:
		if not col_names and data:
			col_names = data[0].keys()

		if exist_header:
			f.write("\t".join(col_names) + "\n")
		for item in data:
			line_str = "\t".join([str(item[x]) for x in col_names])
			f.write(line_str + "\n")



# -*- coding: utf-8 -*-
"""
   Author :        Jesper
   Date：          2020/6/9 20:08
   Description :
   Changed by:
"""

# from numba import jit
import numpy as np
from nlp_utils_ch.utils.logger import log

def vector_similarity(vector_a, vector_b, mode="cosine"):
	sim_score = -1
	if mode == "consine":
		sim_score = cos_sim(vector_a, vector_b)
	else:
		log.info("[ERROR] Not support such mode:{}".format(mode))

	return sim_score



def cos_sim(vector_a, vector_b, decimal_place=6):
	"""
	计算两个向量之间的余弦相似度
	:param vector_a: 向量 a
	:param vector_b: 向量 b
	:return: sim
	"""
	vector_a = np.mat(vector_a)
	vector_b = np.mat(vector_b)
	num = float(vector_a * vector_b.T)
	denom = np.linalg.norm(vector_a) * np.linalg.norm(vector_b)
	cos = num / denom
	# sim = 0.5 + 0.5 * cos
	return round(cos, decimal_place)



# 余弦相似度加速算法
# @jit(nopython=True)
# def cosine_similarity(u: np.ndarray, v: np.ndarray):
#     #     assert(u.shape[0] == v.shape[0])
#     uv = 0
#     uu = 0
#     vv = 0
#     for i in range(u.shape[0]):
#         uv += u[i] * v[i]
#         uu += u[i] * u[i]
#         vv += v[i] * v[i]
#     cos_theta = 0.0
#     if uu != 0 and vv != 0:
#         cos_theta = uv / np.sqrt(uu * vv)
#     return cos_theta
#
#
# @jit(nopython=True)
# def cosine_similarity_batch(u_list: np.ndarray, v: np.ndarray):
#     r = np.zeros(u_list.shape[0])
#     max_score = 0.0
#     for n, u in enumerate(u_list):
#         uv = 0
#         uu = 0
#         vv = 0
#         for i in range(u.shape[0]):
#             uv += u[i] * v[i]
#             uu += u[i] * u[i]
#             vv += v[i] * v[i]
#         cos_theta = 0.0
#         if uu != 0 and vv != 0:
#             cos_theta = uv / np.sqrt(uu * vv)
#             max_score = max(max_score, cos_theta)
#         r[n] = cos_theta
#     return r, max_score


def edit_distance(str1, str2):
    matrix = [[i + j for j in range(len(str2) + 1)] for i in range(len(str1) + 1)]

    for i in range(1, len(str1) + 1):
        for j in range(1, len(str2) + 1):
            if str1[i - 1] == str2[j - 1]:
                d = 0
            else:
                d = 1
            matrix[i][j] = min(matrix[i - 1][j] + 1, matrix[i][j - 1] + 1, matrix[i - 1][j - 1] + d)

    return matrix[len(str1)][len(str2)]



#!user/bin/python
# _*_ coding: utf-8 _*_
# @Author      :   Jesper
# @Time        :   2020/9/29 18:08
# @Description :
from __future__ import print_function, unicode_literals, division

import os
import re
import codecs
import platform

from subprocess import check_output
from tempfile import mkdtemp
from functools import partial

# try:
#     from configparser import ConfigParser
# except ImportError:
#     from ConfigParser import ConfigParser

from pyrouge.utils import log
from pyrouge.utils.file_utils import DirectoryProcessor
from pyrouge.utils.file_utils import verify_dir


class Rouge155(object):
    """
    This is a wrapper for the ROUGE 1.5.5 summary evaluation package.
    This class is designed to simplify the evaluation process by:

        1) Converting summaries into a format ROUGE understands.
        2) Generating the ROUGE configuration file automatically based
            on filename patterns.

    This class can be used within Python like this:

    rouge = Rouge155()
    rouge.system_dir = 'test/systems'
    rouge.model_dir = 'test/models'

    # The system filename pattern should contain one group that
    # matches the document ID.
    rouge.system_filename_pattern = 'SL.P.10.R.11.SL062003-(\d+).html'

    # The model filename pattern has '#ID#' as a placeholder for the
    # document ID. If there are multiple model summaries, pyrouge
    # will use the provided regex to automatically match them with
    # the corresponding system summary. Here, [A-Z] matches
    # multiple model summaries for a given #ID#.
    rouge.model_filename_pattern = 'SL.P.10.R.[A-Z].SL062003-#ID#.html'

    rouge_output = rouge.evaluate()
    print(rouge_output)
    output_dict = rouge.output_to_dict(rouge_ouput)
    print(output_dict)
    ->    {'rouge_1_f_score': 0.95652,
         'rouge_1_f_score_cb': 0.95652,
         'rouge_1_f_score_ce': 0.95652,
         'rouge_1_precision': 0.95652,
        [...]


    To evaluate multiple systems:

        rouge = Rouge155()
        rouge.system_dir = '/PATH/TO/systems'
        rouge.model_dir = 'PATH/TO/models'
        for system_id in ['id1', 'id2', 'id3']:
            rouge.system_filename_pattern = \
                'SL.P/.10.R.{}.SL062003-(\d+).html'.format(system_id)
            rouge.model_filename_pattern = \
                'SL.P.10.R.[A-Z].SL062003-#ID#.html'
            rouge_output = rouge.evaluate(system_id)
            print(rouge_output)

    """

    def __init__(self, rouge_dir=None, rouge_args=None, log_level=None):
        """
        Create a Rouge155 object.

            rouge_dir:  Directory containing Rouge-1.5.5.pl
            rouge_args: Arguments to pass through to ROUGE if you
                        don't want to use the default pyrouge
                        arguments.

        """
        if log_level is None:
                self.log = log.get_global_console_logger()
        else:
                self.log = log.get_global_console_logger(log_level)
        self.__set_dir_properties()
        self._config_file = None
        self._settings_file = self.__get_config_path()
        self.__set_rouge_dir(rouge_dir)
        self.args = self.__clean_rouge_args(rouge_args)
        self._system_filename_pattern = None
        self._model_filename_pattern = None

    def save_home_dir(self):
        config = ConfigParser()
        section = 'pyrouge settings'
        config.add_section(section)
        config.set(section, 'home_dir', self._home_dir)
        with open(self._settings_file, 'w') as f:
            config.write(f)
        self.log.info("Set ROUGE home directory to {}.".format(self._home_dir))

    @property
    def settings_file(self):
        """
        Path of the setttings file, which stores the ROUGE home dir.

        """
        return self._settings_file

    @property
    def bin_path(self):
        """
        The full path of the ROUGE binary (although it's technically
        a script), i.e. rouge_home_dir/ROUGE-1.5.5.pl

        """
        if self._bin_path is None:
            raise Exception(
                "ROUGE path not set. Please set the ROUGE home directory "
                "and ensure that ROUGE-1.5.5.pl exists in it.")
        return self._bin_path

    @property
    def system_filename_pattern(self):
        """
        The regular expression pattern for matching system summary
        filenames. The regex string.

        E.g. "SL.P.10.R.11.SL062003-(\d+).html" will match the system
        filenames in the SPL2003/system folder of the ROUGE SPL example
        in the "sample-test" folder.

        Currently, there is no support for multiple systems.

        """
        return self._system_filename_pattern

    @system_filename_pattern.setter
    def system_filename_pattern(self, pattern):
        self._system_filename_pattern = pattern

    @property
    def model_filename_pattern(self):
        """
        The regular expression pattern for matching model summary
        filenames. The pattern needs to contain the string "#ID#",
        which is a placeholder for the document ID.

        E.g. "SL.P.10.R.[A-Z].SL062003-#ID#.html" will match the model
        filenames in the SPL2003/system folder of the ROUGE SPL
        example in the "sample-test" folder.

        "#ID#" is a placeholder for the document ID which has been
        matched by the "(\d+)" part of the system filename pattern.
        The different model summaries for a given document ID are
        matched by the "[A-Z]" part.

        """
        return self._model_filename_pattern

    @model_filename_pattern.setter
    def model_filename_pattern(self, pattern):
        self._model_filename_pattern = pattern

    @property
    def config_file(self):
        return self._config_file

    @config_file.setter
    def config_file(self, path):
        config_dir, _ = os.path.split(path)
        verify_dir(config_dir, "configuration file")
        self._config_file = path

    def split_sentences(self):
        """
        ROUGE requires texts split into sentences. In case the texts
        are not already split, this method can be used.

        """
        from pyrouge.utils.sentence_splitter import PunktSentenceSplitter
        self.log.info("Splitting sentences.")
        ss = PunktSentenceSplitter()
        sent_split_to_string = lambda s: "\n".join(ss.split(s))
        process_func = partial(
            DirectoryProcessor.process, function=sent_split_to_string)
        self.__process_summaries(process_func)

    @staticmethod
    def convert_summaries_to_rouge_format(input_dir, output_dir):
        """
        Convert all files in input_dir into a format ROUGE understands
        and saves the files to output_dir. The input files are assumed
        to be plain text with one sentence per line.

            input_dir:  Path of directory containing the input files.
            output_dir: Path of directory in which the converted files
                        will be saved.

        """
        DirectoryProcessor.process(
            input_dir, output_dir, Rouge155.convert_text_to_rouge_format)

    @staticmethod
    def convert_text_to_rouge_format(text, title="dummy title"):
        """
        Convert a text to a format ROUGE understands. The text is
        assumed to contain one sentence per line.

            text:   The text to convert, containg one sentence per line.
            title:  Optional title for the text. The title will appear
                    in the converted file, but doesn't seem to have
                    any other relevance.

        Returns: The converted text as string.

        """
        sentences = text.split("\n")
        sent_elems = [
            "<a name=\"{i}\">[{i}]</a> <a href=\"#{i}\" id={i}>"
            "{text}</a>".format(i=i, text=sent)
            for i, sent in enumerate(sentences, start=1)]
        html = """<html>
<head>
<title>{title}</title>
</head>
<body bgcolor="white">
{elems}
</body>
</html>""".format(title=title, elems="\n".join(sent_elems))

        return html

    @staticmethod
    def write_config_static(system_dir, system_filename_pattern,
                            model_dir, model_filename_pattern,
                            config_file_path, system_id=None):
        """
        Write the ROUGE configuration file, which is basically a list
        of system summary files and their corresponding model summary
        files.

        pyrouge uses regular expressions to automatically find the
        matching model summary files for a given system summary file
        (cf. docstrings for system_filename_pattern and
        model_filename_pattern).

            system_dir:                 Path of directory containing
                                        system summaries.
            system_filename_pattern:    Regex string for matching
                                        system summary filenames.
            model_dir:                  Path of directory containing
                                        model summaries.
            model_filename_pattern:     Regex string for matching model
                                        summary filenames.
            config_file_path:           Path of the configuration file.
            system_id:                  Optional system ID string which
                                        will appear in the ROUGE output.

        """
        system_filenames = [f for f in os.listdir(system_dir)]
        system_models_tuples = []

        system_filename_pattern = re.compile(system_filename_pattern)
        for system_filename in sorted(system_filenames):
            match = system_filename_pattern.match(system_filename)
            if match:
                id = match.groups(0)[0]
                model_filenames = Rouge155.__get_model_filenames_for_id(
                    id, model_dir, model_filename_pattern)
                system_models_tuples.append(
                    (system_filename, sorted(model_filenames)))
        if not system_models_tuples:
            raise Exception(
                "Did not find any files matching the pattern {} "
                "in the system summaries directory {}.".format(
                    system_filename_pattern.pattern, system_dir))

        with codecs.open(config_file_path, 'w', encoding='utf-8') as f:
            f.write('<ROUGE-EVAL version="1.55">')
            for task_id, (system_filename, model_filenames) in enumerate(
                    system_models_tuples, start=1):

                eval_string = Rouge155.__get_eval_string(
                    task_id, system_id,
                    system_dir, system_filename,
                    model_dir, model_filenames)
                f.write(eval_string)
            f.write("</ROUGE-EVAL>")

    def write_config(self, config_file_path=None, system_id=None):
        """
        Write the ROUGE configuration file, which is basically a list
        of system summary files and their matching model summary files.

        This is a non-static version of write_config_file_static().

            config_file_path:   Path of the configuration file.
            system_id:          Optional system ID string which will
                                appear in the ROUGE output.

        """
        if not system_id:
            system_id = 1
        if (not config_file_path) or (not self._config_dir):
            self._config_dir = mkdtemp()
            config_filename = "rouge_conf.xml"
        else:
            config_dir, config_filename = os.path.split(config_file_path)
            verify_dir(config_dir, "configuration file")
        self._config_file = os.path.join(self._config_dir, config_filename)
        Rouge155.write_config_static(
            self._system_dir, self._system_filename_pattern,
            self._model_dir, self._model_filename_pattern,
            self._config_file, system_id)
        self.log.info(
            "Written ROUGE configuration to {}".format(self._config_file))

    def evaluate(self, system_id=1, rouge_args=None):
        """
        Run ROUGE to evaluate the system summaries in system_dir against
        the model summaries in model_dir. The summaries are assumed to
        be in the one-sentence-per-line HTML format ROUGE understands.

            system_id:  Optional system ID which will be printed in
                        ROUGE's output.

        Returns: Rouge output as string.

        """
        self.write_config(system_id=system_id)
        options = self.__get_options(rouge_args)
        command = [self._bin_path] + options
        env = os.environ.copy()
        if hasattr(self, "_home_dir") and self._home_dir:
            env['ROUGE_EVAL_HOME'] = self._home_dir
        self.log.info(
            "Running ROUGE with command {}".format(" ".join(command)))
        rouge_output = check_output(command, env=env).decode("UTF-8")
        return rouge_output

    def convert_and_evaluate(self, system_id=1,
                             split_sentences=False, rouge_args=None):
        """
        Convert plain text summaries to ROUGE format and run ROUGE to
        evaluate the system summaries in system_dir against the model
        summaries in model_dir. Optionally split texts into sentences
        in case they aren't already.

        This is just a convenience method combining
        convert_summaries_to_rouge_format() and evaluate().

            split_sentences:    Optional argument specifying if
                                sentences should be split.
            system_id:          Optional system ID which will be printed
                                in ROUGE's output.

        Returns: ROUGE output as string.

        """
        if split_sentences:
            self.split_sentences()
        self.__write_summaries()
        rouge_output = self.evaluate(system_id, rouge_args)
        return rouge_output

    def output_to_dict(self, output):
        """
        Convert the ROUGE output into python dictionary for further
        processing.

        """
        #0 ROUGE-1 Average_R: 0.02632 (95%-conf.int. 0.02632 - 0.02632)
        pattern = re.compile(
            r"(\d+) (ROUGE-\S+) (Average_\w): (\d.\d+) "
            r"\(95%-conf.int. (\d.\d+) - (\d.\d+)\)")
        results = {}
        for line in output.split("\n"):
            match = pattern.match(line)
            if match:
                sys_id, rouge_type, measure, result, conf_begin, conf_end = \
                    match.groups()
                measure = {
                    'Average_R': 'recall',
                    'Average_P': 'precision',
                    'Average_F': 'f_score'
                    }[measure]
                rouge_type = rouge_type.lower().replace("-", '_')
                key = "{}_{}".format(rouge_type, measure)
                results[key] = float(result)
                results["{}_cb".format(key)] = float(conf_begin)
                results["{}_ce".format(key)] = float(conf_end)
        return results

    ###################################################################
    # Private methods

    def __set_rouge_dir(self, home_dir=None):
        """
        Verfify presence of ROUGE-1.5.5.pl and data folder, and set
        those paths.

        """
        if not home_dir:
            self._home_dir = self.__get_rouge_home_dir_from_settings()
        else:
            self._home_dir = home_dir
            self.save_home_dir()
        self._bin_path = os.path.join(self._home_dir, 'ROUGE-1.5.5.pl')
        self.data_dir = os.path.join(self._home_dir, 'data')
        if not os.path.exists(self._bin_path):
            raise Exception(
                "ROUGE binary not found at {}. Please set the "
                "correct path by running pyrouge_set_rouge_path "
                "/path/to/rouge/home.".format(self._bin_path))

    def __get_rouge_home_dir_from_settings(self):
        config = ConfigParser()
        with open(self._settings_file) as f:
            if hasattr(config, "read_file"):
                config.read_file(f)
            else:
                # use deprecated python 2.x method
                config.readfp(f)
        rouge_home_dir = config.get('pyrouge settings', 'home_dir')
        return rouge_home_dir

    @staticmethod
    def __get_eval_string(
            task_id, system_id,
            system_dir, system_filename,
            model_dir, model_filenames):
        """
        ROUGE can evaluate several system summaries for a given text
        against several model summaries, i.e. there is an m-to-n
        relation between system and model summaries. The system
        summaries are listed in the <PEERS> tag and the model summaries
        in the <MODELS> tag. pyrouge currently only supports one system
        summary per text, i.e. it assumes a 1-to-n relation between
        system and model summaries.

        """
        peer_elems = "<P ID=\"{id}\">{name}</P>".format(
            id=system_id, name=system_filename)

        model_elems = ["<M ID=\"{id}\">{name}</M>".format(
            id=chr(65 + i), name=name)
            for i, name in enumerate(model_filenames)]

        model_elems = "\n\t\t\t".join(model_elems)
        eval_string = """
    <EVAL ID="{task_id}">
        <MODEL-ROOT>{model_root}</MODEL-ROOT>
        <PEER-ROOT>{peer_root}</PEER-ROOT>
        <INPUT-FORMAT TYPE="SEE">
        </INPUT-FORMAT>
        <PEERS>
            {peer_elems}
        </PEERS>
        <MODELS>
            {model_elems}
        </MODELS>
    </EVAL>
""".format(
            task_id=task_id,
            model_root=model_dir, model_elems=model_elems,
            peer_root=system_dir, peer_elems=peer_elems)
        return eval_string

    def __process_summaries(self, process_func):
        """
        Helper method that applies process_func to the files in the
        system and model folders and saves the resulting files to new
        system and model folders.

        """
        temp_dir = mkdtemp()
        new_system_dir = os.path.join(temp_dir, "system")
        os.mkdir(new_system_dir)
        new_model_dir = os.path.join(temp_dir, "model")
        os.mkdir(new_model_dir)
        self.log.info(
            "Processing summaries. Saving system files to {} and "
            "model files to {}.".format(new_system_dir, new_model_dir))
        process_func(self._system_dir, new_system_dir)
        process_func(self._model_dir, new_model_dir)
        self._system_dir = new_system_dir
        self._model_dir = new_model_dir

    def __write_summaries(self):
        self.log.info("Writing summaries.")
        self.__process_summaries(self.convert_summaries_to_rouge_format)

    @staticmethod
    def __get_model_filenames_for_id(id, model_dir, model_filenames_pattern):
        pattern = re.compile(model_filenames_pattern.replace('#ID#', id))
        model_filenames = [
            f for f in os.listdir(model_dir) if pattern.match(f)]
        if not model_filenames:
            raise Exception(
                "Could not find any model summaries for the system"
                " summary with ID {}. Specified model filename pattern was: "
                "{}".format(id, model_filenames_pattern))
        return model_filenames

    def __get_options(self, rouge_args=None):
        """
        Get supplied command line arguments for ROUGE or use default
        ones.

        """
        if self.args:
            options = self.args.split()
        elif rouge_args:
            options = rouge_args.split()
        else:
            options = [
                '-e', self._data_dir,
                '-c', 95,
                '-2',
                '-1',
                '-U',
                '-r', 1000,
                '-n', 4,
                '-w', 1.2,
                '-a',
                ]
            options = list(map(str, options))

        options = self.__add_config_option(options)
        return options

    def __create_dir_property(self, dir_name, docstring):
        """
        Generate getter and setter for a directory property.

        """
        property_name = "{}_dir".format(dir_name)
        private_name = "_" + property_name
        setattr(self, private_name, None)

        def fget(self):
            return getattr(self, private_name)

        def fset(self, path):
            verify_dir(path, dir_name)
            setattr(self, private_name, path)

        p = property(fget=fget, fset=fset, doc=docstring)
        setattr(self.__class__, property_name, p)

    def __set_dir_properties(self):
        """
        Automatically generate the properties for directories.

        """
        directories = [
            ("home", "The ROUGE home directory."),
            ("data", "The path of the ROUGE 'data' directory."),
            ("system", "Path of the directory containing system summaries."),
            ("model", "Path of the directory containing model summaries."),
            ]
        for (dirname, docstring) in directories:
            self.__create_dir_property(dirname, docstring)

    def __clean_rouge_args(self, rouge_args):
        """
        Remove enclosing quotation marks, if any.

        """
        if not rouge_args:
            return
        quot_mark_pattern = re.compile('"(.+)"')
        match = quot_mark_pattern.match(rouge_args)
        if match:
            cleaned_args = match.group(1)
            return cleaned_args
        else:
            return rouge_args

    def __add_config_option(self, options):
        return options + ['-m'] + [self._config_file]

    def __get_config_path(self):
        if platform.system() == "Windows":
            parent_dir = os.getenv("APPDATA")
            config_dir_name = "pyrouge"
        elif os.name == "posix":
            parent_dir = os.path.expanduser("~")
            config_dir_name = ".pyrouge"
        else:
            parent_dir = os.path.dirname(__file__)
            config_dir_name = ""
        config_dir = os.path.join(parent_dir, config_dir_name)
        if not os.path.exists(config_dir):
            os.makedirs(config_dir)
        return os.path.join(config_dir, 'settings.ini')


# if __name__ == "__main__":
#     import argparse
#     from utils.argparsers import rouge_path_parser
#
#     parser = argparse.ArgumentParser(parents=[rouge_path_parser])
#     args = parser.parse_args()
#
#     rouge = Rouge155(args.rouge_home)
#     rouge.save_home_dir()



# -*- coding: utf-8 -*-
"""
   Author :        Jesper
   Date：          2020/4/9 14:38
   Description :
   Changed by:
"""
import copy
from nlp_utils_ch.char_judgement.char_judgement import is_number, is_alphabet

def whitespace_tokenize(text, sep=" "):
	"""Runs basic whitespace cleaning and splitting on a piece of text."""
	text = text.strip()
	if not text:
		return []
	tokens = text.split(sep)
	return tokens

def split_string(text, sep=" ", keep_sep=False):
	res_text_list = list()
	for _word in text.split(sep):
		if keep_sep is False:
			if len(_word) != 0:
				res_text_list.append(_word)
		else:
			res_text_list.append(_word)
	return res_text_list

def token_en_number_seq_words(text, save_position=False):
	text = text.replace(" ", "")
	remain_text = ""
	en_num_words = list()
	i = 0
	while i < len(text):
		j = copy.deepcopy(i)
		while j < len(text) and any([is_alphabet(text[j]), is_number(text[j])]):
			j += 1

		if j != i:

			# en_num_position.append([i, j])
			if save_position:
				en_num_words.append({
								"word": text[i:j],
								"position": [i, j]
				})
			else:
				en_num_words.append(text[i:j])
			remain_text += " "
			i = copy.deepcopy(j)

		else:
			remain_text += text[i]
			i += 1
	# print(en_num_words)
	# print(remain_text)
	return en_num_words, remain_text



#!user/bin/python
# _*_ coding: utf-8 _*_
# @Author      :   Jesper
# @Time        :   2021/8/3  17:02
# @Description :


def read_txt_data(file_path, linefeed_keep=False):
	with open(file_path, "r") as f:
		data = f.readlines()
	if linefeed_keep is True:
		return data
	else:
		data = [x.strip() for x in data]
		return data


def save_txt_data(data, file_path, exist_linefeed=False):
	with open(file_path, "w") as f:
		for line in data:
			if exist_linefeed:
				f.write(line)
			else:
				f.write(line + "\n")




#!user/bin/python
# _*_ coding: utf-8 _*_
# @Author      :   Jesper
# @Time        :   2021/10/21  17:48
# @Description :
import time

def print_run_time(func):
	def wrapper(*args, **kw):
		local_time = time.time()
		func(*args, **kw)
		print('current Function [%s] run time is %.2fs' % (func.__name__ ,time.time() - local_time))
	return wrapper



#!user/bin/python
# _*_ coding: utf-8 _*_
# @Author      :   Jesper
# @Time        :   2020/11/11 20:31
# @Description :

import os
current_dir = os.path.dirname(__file__)

stop_symbol_path = os.path.join(current_dir, "../resources/symbol.txt")
stop_ch_path = os.path.join(current_dir, "../resources/stopwords.txt")

stop_symbol_chars = list()
stop_ch_chars = list()


with open(stop_symbol_path, "r") as f:
	for line in f.readlines():
		stop_symbol_chars.append(line.strip())

with open(stop_ch_path, "r") as f:
	for line in f.readlines():
		stop_ch_chars.append(line.strip())



from setuptools import setup, find_packages

setup(
    name="nlp-utils-ch",
    version="1.1.2",
    description="nlp基础工具",
    author="peng.su",
    packages=find_packages(),
    # include_package_data=True,
    package_data={
                '':[
                    'zhconv/zhcdict.json',
                    'resources/*.txt'
                    ],
               
    			},
    install_requires=[
   
      ],
    author_email="alweeq5@163.com",
)



#!user/bin/python
# _*_ coding: utf-8 _*_
# @Author      :   Jesper
# @Time        :   2020/11/18 16:19
# @Description :

import datetime

def get_cur_date():
	now = datetime.datetime.now()
	cur_date = now.strftime("%Y-%m-%d")
	return cur_date

def get_last_nday_date(n=3):
	now = datetime.datetime.now()
	start_datetime = now - datetime.timedelta(days=n)
	start_date = start_datetime.strftime("%Y-%m-%d")
	return start_date




# -*- coding: utf-8 -*-
"""
   Author :        Jesper
   Date：          2020/4/1 18:52
   Description :
   Changed by:
"""
from nlp_utils_ch.char_judgement.char_judgement import is_ch_digit_letter


def find_nearest_phrase(text, word_position):
	# log.debug("Text:{}, word_position:{}".format(text, word_position))
	l = word_position[0]
	r = word_position[1]
	while l >= 0 and is_ch_digit_letter(text[l]):
		l -= 1
	l += 1
	while r < len(text) and is_ch_digit_letter(text[r]):
		r += 1
	# log.debug("Phrase:{}, l:{}, r:{}".format(text[l:r], l, r))
	return text[l:r]



# -*- coding: utf-8 -*-
"""
   Author :        Jesper
   Date：          2020/5/25 17:31
   Description :
   Changed by:
"""


import logging
import threading

global_data = threading.local()

log_level_case = 1
log_level_map = {
				 0: logging.DEBUG,
				 1: logging.INFO,
				 2: logging.ERROR
}
log_level = log_level_map[log_level_case]

class UuidFilter(logging.Filter):
	def filter(self, record):
		uuid = getattr(global_data, 'uuid', None)
		record.uuid = uuid
		return True

log = logging.getLogger(__name__)
handler = logging.StreamHandler()
handler.setLevel(log_level)

formatter = logging.Formatter(
	'%(asctime)s %(levelname)s %(process)d --- [%(threadName)s] %(filename)s:%(lineno)-4d: %(message)s'
)
formatter = logging.Formatter(
	'%(asctime)s %(levelname)s %(process)d --- %(filename)s:%(lineno)-4d: %(message)s'
)

handler.setFormatter(formatter)

log.addHandler(handler)
log.setLevel(log_level)
log.addFilter(UuidFilter())



# -*- coding: utf-8 -*-
"""
   Author :        Jesper
   Date：          2020/2/24 21:34
   Description :
   Changed by:
"""

#  判断是否包含中文
def is_contain_chinese(text):
    for _char in text:
        if '\u4e00' <= _char <= '\u9fa5':
            return True
    return False

def is_contain_en(text):
    for _char in text:
        if is_alphabet(_char):
            return True
    return False


#  判断是否全是中文字符
def is_all_chinese(text):
    for _char in text:
        if not '\u4e00' <= _char <= '\u9fa5':
            return False
    return True

# 判断是否全为英文字符
def is_all_letter(text):
	for _char in text:
		if not is_alphabet(_char):
			return False
	return True


def is_all_number(text):
	for _char in text:
		if not is_number(_char):
			return False
	return True


#   判断是否仅包括中文，数字或者字母（暂时忽略小数）
def is_ch_digit_letter(text):
	for _char in text:
		if not any([is_chinese(_char), is_alphabet(_char), is_number(_char)]):
			return False
	return True

#  包含中文，数字和字母的长度(连续数字和连续字母长度为1)
def len_of_ch_digit_letter(text):
	strs_len = 0
	index = 0
	while index < len(text):
		_char = text[index]
		if is_chinese(_char):
			strs_len +=1
			index += 1
		elif is_alphabet(_char):
			strs_len += 1
			while index < len(text)  and is_alphabet(text[index]): index +=1
		elif is_number(_char):
			strs_len += 1
			while index < len(text)  and is_number(text[index]): index +=1
		else: index += 1
	return strs_len

def len_of_ch(text):
	strs_len = 0
	index = 0
	while index < len(text):
		_char = text[index]
		if is_chinese(_char):
			strs_len += 1
		index += 1
	return strs_len


#  判断一个字符是否是汉字
def is_chinese(uchar):
    if uchar >= u'\u4e00' and uchar <= u'\u9fa5':
        return True
    else:
        return False

#  判断一个字符是否为英文字母
def is_alphabet(uchar):
    if (uchar >= u'\u0041' and uchar <= u'\u005a') or (uchar >= u'\u0061' and uchar <= u'\u007a'):
        return True
    else:
        return False

#  判断一个字符是否为数字
def is_number(uchar):
    if uchar >= u'\u0030' and uchar <= u'\u0039':
        return True
    else:
        return False

#  判断一个字符串是否为可比较数字
def is_comparable_number(word):
	pass








#!user/bin/python
# _*_ coding: utf-8 _*_
# @Author      :   Jesper
# @Time        :   2020/9/16 11:56
# @Description :
import time
import datetime

def timestamp_to_local_time(time_stamp, mode="ms"):
	""":
	mode: ms/s
	"""
	if mode=="ms":
		time_local = time.localtime(time_stamp / 1000)
	else:
		time_local = time.localtime(time_stamp )
	dt = time.strftime("%Y-%m-%d %H:%M:%S", time_local)
	return dt


def to_timestamp(inpput_time):
	d = datetime.datetime.strptime(str(inpput_time), "%Y-%m-%d %H:%M:%S")
	t = d.timetuple()
	time_stamp = int(time.mktime(t))
	return time_stamp
